#ifndef __CONTEXT_H__
#define __CONTEXT_H__
#include "core_import.h"
#include "util.h"
#include "list.h"

#define MAX_SYNC_OBJECT_SIZE_PER_DEVICE  512*4

typedef struct
{
    unsigned int                node_ordinal;
    unsigned int                engine_affinity;
    unsigned int                flags;
}create_context_t;

typedef struct hw_ctxbuf
{
    struct list_head   list_node;

    int index;
    int phys_index;//global context index, adapter function region
    int is_initialized;
    int is_invalid;

    unsigned long long  context_buffer_address;
    unsigned int  *context_buffer;
}hw_ctxbuf_t;

typedef struct gpu_device gpu_device_t;

#define ZX_MAX_PNAME_LEN 128
typedef struct gpu_context
{
    struct list_head   list_node;
    char               pname[ZX_MAX_PNAME_LEN];

    unsigned int       handle;
    int                engine_index;
    gpu_device_t       *device;
    unsigned long      pid;
    unsigned long      tid;

    unsigned long long task_id;
    unsigned long long last_submit_to_sw;
    unsigned long long render_counter;

    unsigned long long wait_delta_check_start; //use for server wait timeout
    unsigned long long wait_delta_check_end;

    int                is_kernel;

    struct os_atomic   *ref_cnt; //dma_ref_cnt

    /* for OGL context, one context can use multi-hwctx.
     * OGL will explictly call add_hwctx to support multi-hwctx,
     * for D3D context == hwctx, no need to add_hwctx, 
     * future will refine vpm code, let video go ogl way to intergate code
     * and delete this per context hw_ctx. 
     * now this only for video client use, OGL use hwctx in device
     */
    hw_ctxbuf_t     hw_ctx;
}gpu_context_t;

struct gpu_device
{
    struct list_head         list_node;

    adapter_t                *adapter;
    void                     *filp;

    unsigned int             handle;
    unsigned long            pid;
    unsigned long            tid;

    char                     pname[ZX_MAX_PNAME_LEN];

    struct os_mutex          *lock;

    /* resource used in this device */
    struct list_head         context_list;
    struct list_head         hwctx_list;
    struct list_head         sync_obj_list;
    struct list_head         allocation_open_instance_list;

    int                      create_num; // device create resource (device, context, hwctx, sync_obj, allocation) num
                                         // since all resource use device ptr and device->lock so device struct and
                                         // lock only can be free when this num = 0, note we also treat device itself as a num,
                                         // when destroy_device called num--, then other resource destroy only can free struct after
                                         // destroy deivce called when num-- lead num == 0. currently only for allocation, device

    int                      task_dropped; //if any context in this device dropped task.
                                           //if so, need force set wait fence sync obj signal to avoid dead lock
                                           //since fence obj signal cmd always put in gerneal dma task.
    
    struct os_atomic         *sync_obj_count; //for limit sync object size per device
    unsigned int             video_seq_index;
};


typedef struct
{
    struct list_head         list_node;        //allocation mode link in device->allocation_open_instance_list
    struct list_head         device_ref_node;  //ref device node linked in allocation->device_ref_list; 
    gpu_device_t             *device;          //reference device,  which device reference this allocation, or which device call this open.
    struct _vidmm_allocation *allocation;
    unsigned int             ref_cnt;          //record how many times this device open this allocation.
}cm_allocation_open_instance_t;

static inline void cm_device_lock(gpu_device_t *device)
{
    zx_mutex_lock(device->lock);
}

static inline void cm_device_unlock(gpu_device_t *device)
{
    zx_mutex_unlock(device->lock);
}

extern gpu_device_t* cm_create_device(adapter_t *adapter, void *process_context);
extern void cm_destroy_device(adapter_t *adapter, gpu_device_t *device);
extern void cm_destroy_remained_device(adapter_t *adapter);
extern gpu_context_t* cm_create_context(gpu_device_t *device, create_context_t *create_context, int is_kernel);
extern void cm_destroy_context(gpu_device_t *device, gpu_context_t *context);
extern void cm_wait_context_idle(gpu_context_t *context);
extern int cm_add_hwctx_buffer(gpu_device_t *device, gpu_context_t *context, unsigned int hw_buf_index);
extern int cm_remove_hwctx_buffer(gpu_device_t *device, gpu_context_t *context, unsigned int hw_buf_index);
extern hw_ctxbuf_t *cm_get_hwctx_buffer(gpu_device_t *device, gpu_context_t *context, unsigned int index);

extern void cm_reset_command_header(gpu_context_t *context);

extern int cm_save(adapter_t *adapter, int need_save_memory);
extern void cm_restore(adapter_t *adapter);

extern int  cm_device_add_reference(gpu_device_t *device, struct _vidmm_allocation *allocation);
extern int  cm_device_del_reference(gpu_device_t *device, struct _vidmm_allocation *allocation);

extern void cm_device_lock_allocation(gpu_device_t *device, struct _vidmm_allocation *allocation);
extern void cm_device_unlock_allocation(gpu_device_t *device, struct _vidmm_allocation *allocation);

extern void cm_device_create_num_inc(gpu_device_t *device);
extern void cm_device_create_num_dec(gpu_device_t *device);

extern void cm_dump_resource(adapter_t *adapter);
extern void cm_dump_device_alloctions(struct os_seq_file *seq_file, adapter_t *adapter,  gpu_device_t *device);
#endif

