#include "zx_adapter.h"
#include "chip_include_e3k.h"
#include "vidsch_engine_e3k.h"
#include "stm_context_e3k.h"
#include "vidsch_blt_e3k.h"
#include "mm_e3k.h"
#include "vidsch_dfs_e3k.h"

extern const DWORD* g_p3DBltCodeSize_E3K ;

static vidsch_chip_func_t engine_gfx_low_chip_func_e3k;
static vidsch_chip_func_t engine_gfx_high_chip_func_e3k;
static vidsch_chip_func_t engine_cs_high_chip_func_e3k;
static vidsch_chip_func_t engine_cs_low0_chip_func_e3k;
static vidsch_chip_func_t engine_cs_low1_chip_func_e3k;
static vidsch_chip_func_t engine_cs_low2_chip_func_e3k;
static vidsch_chip_func_t engine_cs_low3_chip_func_e3k;
static vidsch_chip_func_t engine_vcp_low0_chip_func_e3k;
static vidsch_chip_func_t engine_vcp_low1_chip_func_e3k;
static vidsch_chip_func_t engine_vpp_chip_func_e3k;


//*****************************************************************************
//
//  vidschi_init_hw_settings_e3k
//
// function to init HW MXU register, so that PCIe Ring Buffer can
//  work properly.
//
//*****************************************************************************
void vidschi_init_hw_settings_e3k(adapter_t *adapter)
{
    Reg_Bl_Size                     reg_Bl_Size                   = {0};
    Reg_Bl_Size1                    reg_Bl_Size1                  = {0};
    unsigned char *                 pRegAddr                      = NULL;
    vidmm_segment_t                 *segment                      = NULL;
    heap_t                          *heap                         = NULL;
    Reg_Rb0_Fl2                     KKKRb0                        = {0};
    Reg_Rb1_Fl2                     KKKRb1                        = {0};
    Reg_Rb2_Fl2                     KKKRb2                        = {0};
    Reg_Argument_Fl2                ArgumentBuffer                = {0};
    Reg_Mxu_Dummy_Fence_Addr_Gmi0   reg_Mxu_Dummy_Fence_Addr_Gmi0 = { 0 };
    Reg_Mxu_Dummy_Fence_Addr_Gmi1   reg_Mxu_Dummy_Fence_Addr_Gmi1 = { 0 };
    Reg_Mxu_Dummy_Fence_Addr_Bium   reg_Mxu_Dummy_Fence_Addr_Bium = { 0 };
    engine_share_e3k_t              *share = adapter->private_data;

    heap = vidmm_get_burst_length_heap(adapter);

    pRegAddr = adapter->mmio + MMIO_MXU_START_ADDRESS + Reg_Bl_Size_Offset*4;
    reg_Bl_Size.reg.Buffer_Offset = heap->start >> util_log2(BL_BUFFER_ALIGN);
    reg_Bl_Size.reg.Buffer_Size = util_log2(BL_BUFFER_SIZE >> 23);
    reg_Bl_Size.reg.Invalid_Bl_To_Rf_En = 1;
    zx_write32(pRegAddr, reg_Bl_Size.uint);

    if(adapter->chip_id == CHIP_CHX004)
    {
        pRegAddr = adapter->mmio + MMIO_MXU_START_ADDRESS + Reg_Bl_Size1_Offset*4;
        reg_Bl_Size1.reg.Buffer_Offset = heap->start >> util_log2(BL_BUFFER_ALIGN);
        reg_Bl_Size1.reg.Buffer_Size1 = util_log2(BL_BUFFER_SIZE >> 23);
        reg_Bl_Size1.reg.Invalid_Bl_To_Rf_En = 1;
        zx_write32(pRegAddr, reg_Bl_Size1.uint);
    }
    zx_info("reg_Bl_Size.uint 0x%x readvalue: 0x%x, Invalid BL enable:%d \n", reg_Bl_Size.uint, zx_read32(pRegAddr), reg_Bl_Size.reg.Invalid_Bl_To_Rf_En);

    if(adapter->chip_id == CHIP_CHX004)
    {
        reg_Mxu_Dummy_Fence_Addr_Gmi0.reg.Address = (share->dummyfence_phy[1] >> 6);
        pRegAddr = adapter->mmio + MMIO_MXU_START_ADDRESS + Reg_Mxu_Dummy_Fence_Addr_Gmi0_Offset * 4;
        zx_write32(pRegAddr, reg_Mxu_Dummy_Fence_Addr_Gmi0.uint);
        zx_debug("reg_Mxu_Dummy_Fence_Addr_Gmi0.reg.Address: %x\n", reg_Mxu_Dummy_Fence_Addr_Gmi0.reg.Address);

        reg_Mxu_Dummy_Fence_Addr_Gmi1.reg.Address = (share->dummyfence_phy[1] >> 6) + 4;
        pRegAddr = adapter->mmio + MMIO_MXU_START_ADDRESS + Reg_Mxu_Dummy_Fence_Addr_Gmi1_Offset * 4;
        zx_write32(pRegAddr, reg_Mxu_Dummy_Fence_Addr_Gmi1.uint);
        zx_debug("reg_Mxu_Dummy_Fence_Addr_Gmi1.reg.Address: %x\n", reg_Mxu_Dummy_Fence_Addr_Gmi1.reg.Address);

        reg_Mxu_Dummy_Fence_Addr_Bium.reg.Address = (share->dummyfence_phy[0] >> 6);
        pRegAddr = adapter->mmio + MMIO_MXU_START_ADDRESS + Reg_Mxu_Dummy_Fence_Addr_Bium_Offset * 4;
        zx_write32(pRegAddr, reg_Mxu_Dummy_Fence_Addr_Bium.uint);
	zx_debug("reg_Mxu_Dummy_Fence_Addr_Bium.reg.Address: %x\n", reg_Mxu_Dummy_Fence_Addr_Bium.reg.Address);
    }

    KKKRb0.reg.Size             = RING_BUFFER_SIZE_E3K >> 16;
    KKKRb1.reg.Size             = RING_BUFFER_SIZE_E3K >> 16;
    KKKRb2.reg.Size             = RING_BUFFER_SIZE_E3K >> 16;
    ArgumentBuffer.reg.Size     = KKK_ARGUMENT_BUFFER_SIZE >> 16;

    pRegAddr = adapter->mmio + MMIO_MXU_START_ADDRESS + Reg_Rb0_Fl2_Offset*4;
    zx_write32(pRegAddr, KKKRb0.uint);
    pRegAddr = adapter->mmio + MMIO_MXU_START_ADDRESS + Reg_Rb1_Fl2_Offset*4;
    zx_write32(pRegAddr, KKKRb1.uint);
    pRegAddr = adapter->mmio + MMIO_MXU_START_ADDRESS + Reg_Rb2_Fl2_Offset*4;
    zx_write32(pRegAddr, KKKRb2.uint);
    pRegAddr = adapter->mmio + MMIO_MXU_START_ADDRESS + Reg_Argument_Fl2_Offset*4;
    zx_write32(pRegAddr, ArgumentBuffer.uint);

    //Note: only discrete card has onchip miu
    if(adapter->chip_id == CHIP_ELT3K)
    {
        // TODO: secure range regs is 64k aligned, needs align shift?? seems yes!!
        //per Xinzhao, secure regs can't be access through mmio, pending for refine to PMP.
        //BL set to secure range 0
        //Per Baldwin, driver should cfg each miu about secure info.
        int i = 0;

        for (i = 0; i < adapter->hw_caps.miu_channel_num; i++)
        {
            zx_write32(adapter->mmio + MIU0_RANGE_BASE_ADDRESS + i*0x1000,     ((heap->start >> 16) << 12));
            zx_write32(adapter->mmio + MIU0_RANGE_BASE_ADDRESS + i*0x1000 + 4, (((heap->start + heap->size) >> 16) << 12));

            //zx_info("BL:exp:%x,read:%x, exp:%x,read:%x\n", ((heap->start >> 16) << 12), zx_read32(adapter->mmio + MIU0_RANGE_BASE_ADDRESS + i*0x1000), (((heap->start + heap->size) >> 16) << 12), zx_read32(adapter->mmio + MIU0_RANGE_BASE_ADDRESS + i*0x1000 +4));

            if(adapter->hw_caps.secure_range_enable)
            {
                //set secure range1, secure range
                segment = vidmm_get_segment_by_id(adapter, SEGMENT_ID_SECURE_RANGE_E3K);
                zx_write32(adapter->mmio + MIU0_RANGE_BASE_ADDRESS + i*0x1000 + 1*8,     ((segment->gpu_vm_start >> 16) << 12));
                zx_write32(adapter->mmio + MIU0_RANGE_BASE_ADDRESS + i*0x1000 + 1*8 +4, (((segment->gpu_vm_start + segment->gpu_vm_size) >> 16) << 12));
                zx_info("SEC[1]exp:%x,read:%x, exp:%x,read:%x\n", ((segment->gpu_vm_start >> 16) << 12), zx_read32(adapter->mmio + MIU0_RANGE_BASE_ADDRESS + i*0x1000 +8 ), (((segment->gpu_vm_start + segment->gpu_vm_size) >> 16) << 12), zx_read32(adapter->mmio + MIU0_RANGE_BASE_ADDRESS + i*0x1000 +0xc));

                //set secure range2, non-secure range
                segment = vidmm_get_segment_by_id(adapter, SEGMENT_ID_SECURE_RANGE_E3K_1);
                zx_write32(adapter->mmio + MIU0_RANGE_BASE_ADDRESS + i*0x1000 + 2*8,     ((segment->gpu_vm_start >> 16) << 12));
                zx_write32(adapter->mmio + MIU0_RANGE_BASE_ADDRESS + i*0x1000 + 2*8 +4, (((segment->gpu_vm_start + segment->gpu_vm_size) >> 16) << 12));
                zx_info("SEC[2]exp:%x,read:%x, exp:%x,read:%x\n", ((segment->gpu_vm_start >> 16) << 12), zx_read32(adapter->mmio + MIU0_RANGE_BASE_ADDRESS + i*0x1000 +2*8 ), (((segment->gpu_vm_start + segment->gpu_vm_size) >> 16) << 12), zx_read32(adapter->mmio + MIU0_RANGE_BASE_ADDRESS + i*0x1000 + 2*8 +4));

                //set scramble bits[8,15]: range0~7; bit[0]:1 means enable scramble. Rang0 for BL, should not enable scramble
                zx_write32(adapter->mmio + SECURE_RANGE_ENCRYPTEN_ADDRESS + i*0x1000, ((1 << 9) | (1 << 0)));

                 // set the scramble pattern for later scramble/descramble use.
                zx_write32(adapter->mmio + SECURE_RANGE_SCRAMBLE_PATTEN_ADDRESS + i*0x1000   , 0xFFFFFFFF);
                zx_write32(adapter->mmio + SECURE_RANGE_SCRAMBLE_PATTEN_ADDRESS + i*0x1000 + 0x4, 0xFFFFFFFF);
                zx_write32(adapter->mmio + SECURE_RANGE_SCRAMBLE_PATTEN_ADDRESS + i*0x1000 + 0x8, 0xFFFFFFFF);
                zx_write32(adapter->mmio + SECURE_RANGE_SCRAMBLE_PATTEN_ADDRESS + i*0x1000 + 0xc, 0xFFFFFFFF);
            }
        }
    }

}

/*
//E3K:
assign SWRST[7:0] = REG_CRA3 | REG8524[7:0];
assign SWRST[15:8] = REG_CRA4 | REG8524[15:8];
assign SWRST[23:16] = REG_CRA5 | REG8524[23:16];
assign SWRST[31:24] = REG_CRA6 | REG8524[31:24];
assign SWRST[39:32] = REG_CRB4 | REG85B4[7:0];
assign SWRST[47:40] = REG_CRB5 | REG85B4[15:8];
assign SWRST[55:48] = REG_CRB6 | REG85B4[23:16];
assign SWRST[63:56] = REG_CRB7 | REG85B4[31:24];

The following is the detail for SWRST[23:0]
MIU_SwRst        = SWRST[1]  | SWRST[0];
For_3D_SwRst     = SWRST[2]  | SWRST[0];
For_3D1_SwRst    = SWRST[3]  | SWRST[0];
For_3D2_SwRst    = SWRST[4]  | SWRST[0];
For_3D3_SwRst    = SWRST[5]  | SWRST[0];
VPP_SwRst        = SWRST[6]  | SWRST[0];
ZXVD0_SwRst      = SWRST[7]  | SWRST[0];
ZXVD1_SwRst      = SWRST[8]  | SWRST[0];
mmio_timer_SwRst = SWRST[9]  | SWRST[0];
m0pll_div_SwRst  = SWRST[10] | SWRST[0];
m1pll_div_SwRst  = SWRST[11] | SWRST[0];
m2pll_div_SwRst  = SWRST[12] | SWRST[0];
epll_div_SwRst   = SWRST[13] | SWRST[0];
vpll_div_SwRst   = SWRST[14] | SWRST[0];
IGA1_SwRst       = SWRST[16] | SWRST[0];
IGA2_SwRst       = SWRST[17] | SWRST[0];
IGA3_SwRst       = SWRST[18] | SWRST[0];

If hang, just need reset 3D,3D1,3D2,3D3,VPP,ZXVD0,ZXVD1,so set 8524 to 0x01FC.
or we can set CRA3 and CRA4, the mmio offset is 0x8aa3,0x8aa4.

CHX004:
PMU_PART_SW_RESET_set
	[29]	MXU1_ADB_RSTB
	[28]	MXU0_ADB_RSTB
	[27]	REG_RSTB_tmp
	[26]	IGA1_RSTB_tmp
	[25]	IGA2_RSTB_tmp
	[24]	IGA3_RSTB_tmp
	[23]	HOT_WIRE_RESETB_tmp
	[22]	PMP_RESETB
	[21]	BIU_RESETB
	[20]	DIU_RESETB
	[19]	MXU1_RESETB
	[18]	CSP_MXU0_RESETB
	[17]	L2_RESETB_tmp
	[16]	POOL_RESETB_tmp
	[15]	SGTBE_RESETB_tmp
	[14]	SPTFE_RESETB_tmp
	[13]	EUC0_RESETB_tmp
	[12]	FFU0_RESETB_tmp
	[11]	TU0_RESETB_tmp
	[10]	PEA0_RESETB_tmp
	[9]	PEB0_RESETB_tmp
	[8]	EUC1_RESETB_tmp
	[7]	FFU1_RESETB_tmp
	[6]	TU1_RESETB_tmp
	[5]	PEA1_RESETB_tmp
	[4]	PEB1_RESETB_tmp
	[3]	S3VD0_RESETB_tmp
	[2]	S3VD1_RESETB_tmp
	[1]	VPP_RESETB_tmp
	[0]	GFX_TOP_RESETB
*/

void vidschi_reset_adapter_e3k(adapter_t *adapter)
{
    if(adapter->chip_id == CHIP_ELT3K)
    {
        Reg_HWReset_E3K  dwResetHW = {0};
        volatile unsigned int	 i, j;

        //zx_disp_wait_idle(adapter->disp_info);

        dwResetHW.uint = 0x01FC;
        zx_write32(adapter->mmio + 0x8524, dwResetHW.uint);

        //wait a while
        j=0;
        for (i=0;i<0x10000;i++){j++;};

        dwResetHW.uint = 0; //clear reset value.
        zx_write32(adapter->mmio + 0x8524, dwResetHW.uint);

        //wait a while
        j=0;
        for (i=0;i<0x10000;i++) {j++;};
    }
    else if(adapter->chip_id == CHIP_CHX004)
    {
        unsigned int read_value = 0;
        Reg_HWReset_004  dwResetHW = {0};

        zx_disp_wait_idle(adapter->disp_info);

        read_value = zx_read32(adapter->mmio + 0x6882c); // PMU_SW_RST_AUTO_RLS
        zx_write32(adapter->mmio + 0x6882c, (read_value & 0xfffffffe));//PMU_SW_RST_AUTO_RLS

        dwResetHW.uint = 0x300C00FF;
        zx_write32(adapter->mmio + 0x68808, dwResetHW.uint);//PMU_PART_SW_RESET_set

        zx_udelay(20);

        zx_write32(adapter->mmio + 0x68830, dwResetHW.uint);//PMU_PART_SW_RESET_set

        while(zx_read32(adapter->mmio + 0x68808));

        zx_udelay(20);

        if (adapter->pm_caps.pwm_mode)
        {
            Reg_Pwr_Mgr_Status1 reg_status1;
            int delay_times = 2000 * 100; // 2000 * 100 * 1 us = 200ms

            reg_status1.uint = zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Pwr_Mgr_Status1_Offset * 4);

            while(reg_status1.reg.Csp_Pwm_Central_Pwron_Be == 0 ||
                  reg_status1.reg.Csp_Pwm_Slc0_Pwron_Be == 0 ||
                  reg_status1.reg.Csp_Pwm_Slc1_Pwron_Be == 0 ||
                  reg_status1.reg.Csp_Pwm_S3vd0_Pwron_Be == 0 ||
                  reg_status1.reg.Csp_Pwm_S3vd1_Pwron_Be == 0 ||
                  reg_status1.reg.Csp_Pwm_Vpp_Pwron_Be == 0)
            {
                  delay_times--;
                  if (!delay_times)
                  {
                      zx_warning("cannot power on Be !!! reg_status1=0x%x\n", reg_status1);
                      break;
                  }
                  zx_udelay(1);
                  reg_status1.uint = zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Pwr_Mgr_Status1_Offset * 4);
            }

            // reset again
            read_value = zx_read32(adapter->mmio + 0x6882c); // PMU_SW_RST_AUTO_RLS
            zx_write32(adapter->mmio + 0x6882c, (read_value & 0xfffffffe));//PMU_SW_RST_AUTO_RLS

            dwResetHW.uint = 0x300C00FF;
            zx_write32(adapter->mmio + 0x68808, dwResetHW.uint);//PMU_PART_SW_RESET_set

            zx_udelay(20);

            zx_write32(adapter->mmio + 0x68830, dwResetHW.uint);//PMU_PART_SW_RESET_set

            while(zx_read32(adapter->mmio + 0x68808));
        }
    }

    {
    	vidschedule_t *schedule = adapter->schedule;
        EngineSatus_e3k engine_status = {0};
        unsigned int *status = (unsigned int *)&engine_status;
        int i;

        for( i = 0; i <7; i++)
        {
            *(status + i) = zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS + (Reg_Block_Busy_Bits_Top_Offset + i)*4);
        }

        zx_info("after reset......\n");
        zx_info("current engine Status: %08x,%08x,%08x,%08x,%08x,%08x,%08x\n",
		engine_status.Top.uint,engine_status.Gpc0_0.uint,engine_status.Gpc0_1.uint,
		engine_status.Gpc1_0.uint,engine_status.Gpc1_1.uint,engine_status.Gpc2_0.uint,engine_status.Gpc2_1.uint);
        zx_info("done......\n");
    }
}

void vidschi_map_e3k(adapter_t *adapter);

//*****************************************************************************
//
// vidschi_init_lookup_table_e3k
//
// Init ring buffer lookup table, this table map from node_ordinal and
// engine_affinity to specific engine index
//
//*****************************************************************************
void vidschi_init_lookup_table_e3k(adapter_t  *adapter, vidsch_query_private_t *query)
{
    int i, j, total_size = 0;
    unsigned int     hw_ctx_buffer_size = (adapter->chip_id == CHIP_ELT3K)? CONTEXT_BUFFER_SIZE : CONTEXT_BUFFER_SIZE_CHX004;

    query->engine_count = RB_NUM;

    if(adapter->hw_caps.local_only)
    {
        query->pcie_segment_id         = SEGMENT_ID_LOCAL_E3K;
        query->local_segment_id        = SEGMENT_ID_LOCAL_E3K;
#if defined(__mips64__)
        //PCIE cfg 0x68[14:12] defualt value is 010b, which means 512 Bytes maximum Read Request size
        //mips system BIOS(loonson PMON) modify this value to 000b, which means 128 Bytes maximum Read Request size
        //if fence buffer is in LOCAL, BIU will receive 8*256 bits Dummy Fence Read Request, which exceed 128 Bytes,
        //and BIU should split this Dummy Fence Request to two Dummy Fence Request,
        //but BIU has a bug which leads to the second Dummy Fence Read Request as a normal request
        query->fence_buffer_segment_id = SEGMENT_ID_GART_UNSNOOPABLE_E3K;
#else
        query->fence_buffer_segment_id = SEGMENT_ID_LOCAL_E3K;
#endif
    }
    else
    {
        query->local_segment_id        = SEGMENT_ID_LOCAL_E3K;

        if (adapter->hw_caps.snoop_only)
        {
            query->fence_buffer_segment_id = SEGMENT_ID_GART_SNOOPABLE_E3K;
            query->pcie_segment_id         = SEGMENT_ID_GART_SNOOPABLE_E3K;
        }
        else
        {
#if ENABLE_UNSNOOPABLE_WROKAROUND
            query->fence_buffer_segment_id = SEGMENT_ID_GART_SNOOPABLE_E3K;
            query->pcie_segment_id         = SEGMENT_ID_GART_SNOOPABLE_E3K;
#else
            query->fence_buffer_segment_id = SEGMENT_ID_GART_UNSNOOPABLE_E3K;
            query->pcie_segment_id         = SEGMENT_ID_GART_UNSNOOPABLE_E3K;
#endif
        }

    }

    for ( i = 0; i < query->engine_count; i++)
    {
        //query->local_memory_size[i]= HW_CONTEXT_SIZE_E3K; // for hw context buffer start addr align requirement
        //query->local_memory_size[i]  += E3K_3DBLT_RESERVED_SIZE;  //kernel 3D blt buffer size
        query->engine_hw_queue_size[i] = E3K_ENGINE_HW_QUEUE_SIZE;

        switch(i)
        {
        case RB_INDEX_GFXL:
            adapter->engine_index_table[i].node_ordinal    = 0;
            adapter->engine_index_table[i].engine_affinity = 1;

            query->engine_caps[i]      = ENGINE_CAPS_PAGING |
                                                    ENGINE_CAPS_2D_GRAPHICS | ENGINE_CAPS_3D_GRAPHICS;
            query->engine_ctrl[i]      = ENGINE_CTRL_THREAD_ENABLE;
            query->dma_buffer_size[i]  = 2*1024*1024;
            query->pcie_memory_size[i] = RING_BUFFER_SIZE_E3K;
            query->local_memory_size[i]= (hw_ctx_buffer_size * (HW_CONTEXT_COUNT_E3K + 1))
                                                                + BEGIN_END_BUF_SIZE_E3K;
            total_size += _3DBLT_DATA_RESERVE_SIZE;

            query->local_memory_size[i] += total_size;
            query->engine_func[i]        = &engine_gfx_low_chip_func_e3k;
            break;

        case RB_INDEX_GFXH:
            adapter->engine_index_table[i].node_ordinal    = 1;
            adapter->engine_index_table[i].engine_affinity = 1;

            query->engine_caps[i]      = ENGINE_CAPS_2D_GRAPHICS | ENGINE_CAPS_3D_GRAPHICS;
            query->engine_ctrl[i]      = ENGINE_CTRL_THREAD_ENABLE;
            query->dma_buffer_size[i]  = 1*1024*1024;
            query->pcie_memory_size[i] = RING_BUFFER_SIZE_E3K;

            query->local_memory_size[i]= (hw_ctx_buffer_size * (HW_CONTEXT_COUNT_E3K + 1));
            query->engine_func[i]      = &engine_gfx_high_chip_func_e3k;
            break;

        case RB_INDEX_CSH:
            adapter->engine_index_table[i].node_ordinal    = 2;
            adapter->engine_index_table[i].engine_affinity = 1;

            query->engine_caps[i]      = ENGINE_CAPS_CS;
            query->engine_ctrl[i]      = ENGINE_CTRL_DISABLE;
            query->dma_buffer_size[i]  = 1*1024*1024;
            query->pcie_memory_size[i] = RING_BUFFER_SIZE_E3K;
            query->local_memory_size[i]= 0;
            query->engine_func[i]      = &engine_cs_high_chip_func_e3k;
            break;

        case RB_INDEX_CSL0:
            adapter->engine_index_table[i].node_ordinal    = 3;
            adapter->engine_index_table[i].engine_affinity = 1;

            query->engine_caps[i]      = ENGINE_CAPS_CS;
            query->engine_ctrl[i]      = ENGINE_CTRL_DISABLE;
            query->dma_buffer_size[i]  = 1*1024*1024;
            query->pcie_memory_size[i] = RING_BUFFER_SIZE_E3K;
            query->local_memory_size[i]= 0;
            query->engine_func[i]      = &engine_cs_low0_chip_func_e3k;
            break;

        case RB_INDEX_CSL1:
            // Paging RB visible in KMD and KMD runtime
            adapter->engine_index_table[i].node_ordinal    = 4;
            adapter->engine_index_table[i].engine_affinity = 1;

            query->engine_caps[i]      =  ENGINE_CAPS_CS;
            query->engine_ctrl[i]      =  ENGINE_CTRL_DISABLE;
            query->dma_buffer_size[i]  = 1*1024*1024;
            query->pcie_memory_size[i] = RING_BUFFER_SIZE_E3K;
            query->engine_func[i]      =  &engine_cs_low1_chip_func_e3k;
            break;

        case RB_INDEX_CSL2:
            adapter->engine_index_table[i].node_ordinal    = 5;
            adapter->engine_index_table[i].engine_affinity = 1;

            query->engine_caps[i]      =  ENGINE_CAPS_CS;
            query->engine_ctrl[i]      =  ENGINE_CTRL_DISABLE;
            query->dma_buffer_size[i]  = 1*1024*1024;
            query->pcie_memory_size[i] = RING_BUFFER_SIZE_E3K;
            query->local_memory_size[i]= 0;
            query->engine_func[i]      = &engine_cs_low2_chip_func_e3k;
            break;

        case RB_INDEX_CSL3:
            adapter->engine_index_table[i].node_ordinal    = 6;
            adapter->engine_index_table[i].engine_affinity = 1;

            query->engine_caps[i]      = ENGINE_CAPS_CS ;
            query->engine_ctrl[i]      = ENGINE_CTRL_DISABLE;
            query->dma_buffer_size[i]  = 1*1024*1024;
            query->pcie_memory_size[i] = RING_BUFFER_SIZE_E3K;
            query->local_memory_size[i]= 0;
            query->engine_func[i]      = &engine_cs_low3_chip_func_e3k;
            break;

        case RB_INDEX_VCP0:
            adapter->engine_index_table[i].node_ordinal    = 7;
            adapter->engine_index_table[i].engine_affinity = 1;

            query->engine_caps[i]      = ENGINE_CAPS_VIDEO_DECODE | ENGINE_CAPS_VIDEO_ENCODE | ENGINE_CAPS_NO_HWCTX;
            query->engine_ctrl[i]      = ENGINE_CTRL_THREAD_ENABLE;
            query->dma_buffer_size[i]  = 1*1024*1024;
            query->pcie_memory_size[i] = RING_BUFFER_SIZE_E3K;
            query->local_memory_size[i]= VIDEO_BRIDGE_BUF_SIZE;
            query->engine_func[i]      = &engine_vcp_low0_chip_func_e3k;
            break;
        case RB_INDEX_VCP1:
            adapter->engine_index_table[i].node_ordinal    = 8;
            adapter->engine_index_table[i].engine_affinity = 1;

            query->engine_caps[i]      = ENGINE_CAPS_VIDEO_DECODE | ENGINE_CAPS_VIDEO_ENCODE | ENGINE_CAPS_NO_HWCTX;
            query->engine_ctrl[i]      = ENGINE_CTRL_THREAD_ENABLE;
            query->dma_buffer_size[i]  = 1*1024*1024;
            query->pcie_memory_size[i] = RING_BUFFER_SIZE_E3K;
            query->local_memory_size[i]= VIDEO_BRIDGE_BUF_SIZE;
            query->engine_func[i]      = &engine_vcp_low1_chip_func_e3k;
            break;
        case RB_INDEX_VPP:
            adapter->engine_index_table[i].node_ordinal    = 9;
            adapter->engine_index_table[i].engine_affinity = 1;

            query->engine_caps[i]      = ENGINE_CAPS_VIDEO_VPP | ENGINE_CAPS_NO_HWCTX;
            query->engine_ctrl[i]      = ENGINE_CTRL_THREAD_ENABLE;
            query->dma_buffer_size[i]  = 1*1024*1024;
            query->pcie_memory_size[i] = RING_BUFFER_SIZE_E3K;
            query->local_memory_size[i]= 0;
            query->engine_func[i]      = &engine_vpp_chip_func_e3k;
            break;

        default:
            zx_assert(0, NULL);
            break;
        }

        if(adapter->hw_caps.local_only)
        {
            query->dma_segment[i]      = SEGMENT_ID_LOCAL_E3K;
        }
        else
        {
            query->dma_segment[i]      = SEGMENT_ID_GART_SNOOPABLE_E3K;

            if(! adapter->hw_caps.snoop_only)
            {
#if !ENABLE_UNSNOOPABLE_WROKAROUND
                query->dma_segment[i]      = SEGMENT_ID_GART_UNSNOOPABLE_E3K;
#endif
            }
        }
    }

    if(adapter->ctl_flags.hang_dump)
    {
        engine_share_e3k_t      *share     = adapter->private_data;
        unsigned int            segment_id        = SEGMENT_ID_LOCAL_E3K;
        vidmm_segment_t         *segment          = vidmm_get_segment_by_id(adapter, segment_id);

        /* reserve ring buffer for hang */
        share->ring_buffer_for_hang   = vidmm_allocate_segment_memory(adapter, segment_id, E3K_RINGBUFFER_FOR_HANG_SIZE, 1);
        zx_info("ringbuffer: gpu virt=%x, size=%x\n", share->ring_buffer_for_hang->gpu_virt_addr, share->ring_buffer_for_hang->list_node->aligned_size);

        /* reserve context buffer for hang */
        share->context_buffer_for_hang   = vidmm_allocate_segment_memory(adapter, segment_id, E3K_CONTEXTBUFFER_FOR_HANG_SIZE, 1);
        zx_info("contextbuffer: gpu virt=%x, size=%x\n", share->context_buffer_for_hang->gpu_virt_addr, share->context_buffer_for_hang->list_node->aligned_size);

        /* reserve dma buffer for hang */
        share->dma_buffer_for_hang   = vidmm_allocate_segment_memory(adapter, segment_id, E3K_DMABUFFER_FOR_HANG_SIZE, 1);
        zx_info("dmabuffer: gpu virt=%x, size=%x\n", share->dma_buffer_for_hang->gpu_virt_addr, share->dma_buffer_for_hang->list_node->aligned_size);

        /* reserve pcie buffer for copy cpu invisble segment out*/
        segment_id = SEGMENT_ID_GART_UNSNOOPABLE_E3K;
        share->transfer_buffer_for_hang = vidmm_allocate_segment_memory(adapter, segment_id, E3K_TRANSFERBUFFER_FOR_HANG_SIZE, 0);
        zx_info("transfer buffer for hang: gpu virt=%x, size=%x\n", share->transfer_buffer_for_hang->gpu_virt_addr, share->transfer_buffer_for_hang->list_node->aligned_size);

        vidschi_map_e3k(adapter);
    }
}

static int vidmmi_get_dummyfence_addr(void *arg, int pg_start, int pg_cnt, unsigned long long dma_addr)
{
    *(unsigned long long*)arg = dma_addr;

    return 1;
}

int vidschi_init_engine_share_e3k(adapter_t *adapter)
{
    engine_share_e3k_t  *share = zx_calloc(sizeof(engine_share_e3k_t));

    share->lock = zx_create_spinlock(0);

    adapter->private_data = share;

    //allocate burst length buffer in local reserved
    {
        heap_t *heap = NULL;
        unsigned long long    heap_size, heap_start;

        heap = vidmm_get_burst_length_heap(adapter);

        zx_assert(BL_SLICE_ALIGNMENT <= adapter->os_page_size, "slot buffer alignment is less than page size alignment.");

        share->bl_buffer = vidmm_allocate_segment_memory(adapter, SEGMENT_ID_LOCAL_E3K, (BL_BUFFER_SIZE + BL_BUFFER_ALIGN), 1);

        zx_info("bl_buffer: gpu_virt_addr=0x%llx, size=0x%llx\n", share->bl_buffer->gpu_virt_addr, share->bl_buffer->list_node->aligned_size);

        heap_start = (share->bl_buffer->gpu_virt_addr + BL_BUFFER_ALIGN - 1) & (~(BL_BUFFER_ALIGN-1));//align
        heap_size  = BL_BUFFER_SIZE;

        zx_info("bl_buffer aligned: gpu_virt_addr=0x%llx, size=0x%llx\n", heap_start, heap_size);

        heap_init(heap, 1, heap_start, heap_size, BL_SLICE_ALIGNMENT);
    }

    {
        vidmm_map_flags_t flags = {0};
        unsigned int segment_id =
        adapter->hw_caps.local_only ? SEGMENT_ID_LOCAL_E3K: (adapter->hw_caps.snoop_only ? SEGMENT_ID_GART_SNOOPABLE_E3K : SEGMENT_ID_GART_UNSNOOPABLE_E3K);

        flags.mem_space = ZX_MEM_KERNEL;
        flags.cache_type = ZX_MEM_UNCACHED;

        share->flush_fifo_buffer = vidmm_allocate_segment_memory(adapter, segment_id, 0x1000, 0);

        vidmm_map_segment_memory(adapter, NULL, share->flush_fifo_buffer, &flags);

        zx_info("flush_fifo_buffer: gpu_virt_addr=%x, kmd_virt_addr=%p\n", share->flush_fifo_buffer->gpu_virt_addr, share->flush_fifo_buffer->vma->virt_addr);
    }

    {
       alloc_pages_flags_t alloc_flags = {0};
       alloc_flags.dma32 = TRUE;
       alloc_flags.need_zero = TRUE;
       alloc_flags.fixed_page = TRUE;
       alloc_flags.need_dma_map = 1;
       share->dummyfence[0] = zx_allocate_pages_memory(adapter->os_device.pdev, adapter->os_page_size, adapter->os_page_size, alloc_flags);
       zx_pages_memory_for_each_continues(share->dummyfence[0], &share->dummyfence_phy[0], vidmmi_get_dummyfence_addr);

       alloc_flags.need_dma_map = 0;
       share->dummyfence[1] = zx_allocate_pages_memory(adapter->os_device.pdev, adapter->os_page_size, adapter->os_page_size, alloc_flags);
       zx_pages_memory_for_each_continues(share->dummyfence[1], &share->dummyfence_phy[1], vidmmi_get_dummyfence_addr);

       zx_info("dummy_fence_snoop:      phy_addr=0x%llx\n", share->dummyfence_phy[0]);
       zx_info("dummy_fence_nonsnoop:   phy_addr=0x%llx\n", share->dummyfence_phy[1]);
    }
    return S_OK;
}

int vidschi_init_engine_schedule_profile_e3k(adapter_t *adapter, vidsch_query_private_t *query)
{
    int i;

    for ( i = 0; i < query->engine_count; i++)
    {
        switch(i)
        {
        case RB_INDEX_GFXL:
        //case RB_INDEX_GFXH:
        case RB_INDEX_VPP:
            query->schedule_serialize[i] = 1;
            break;
        default:
            query->schedule_serialize[i] = 0;
            break;
        }
    }

    return 0;
}

int vidsch_query_chip(adapter_t *adapter, vidsch_query_private_t *query)
{
    //enable reset for qt/HAPS test
    if(adapter->ctl_flags.run_on_qt)
    {
        //vidschi_reset_adapter_e3k(adapter);
    }

    vidschi_init_engine_share_e3k(adapter);

    vidschi_init_lookup_table_e3k(adapter, query);

    vidschi_init_hw_settings_e3k(adapter);

    vidschi_init_dvfs_setting_004(adapter);

    vidschi_init_engine_schedule_profile_e3k(adapter, query);

    return S_OK;
}


void vidsch_get_set_reg_e3k(adapter_t *adapter, zx_query_info_t *info)
{
    unsigned long long time_stamp_hi, time_stamp_lo;
    unsigned int *buf;
    int i=0;
    unsigned long flags = 0;

    if (adapter->pm_caps.pll_control)
    {
        flags = zx_spin_lock_irqsave(adapter->schedule->pll_lock);
        if (!adapter->schedule->pll_on)
        {
            vidsch_pll_on_off_e3k(adapter, TRUE);
        }
        zx_del_timer(adapter->schedule->timer);
    }

    switch(info->type)
    {
        case ZX_QUERY_GPU_TIME_STAMP:
#ifdef __aarch64__
            unsigned long long time_stamp_hi, time_stamp_lo;
            if (adapter->chip_id == CHIP_ELT3K)
            {
                time_stamp_lo = zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS  + Reg_Csp_Ms_Total_Gpu_Timestamp_Offset * 4);
                time_stamp_hi = zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS  + Reg_Csp_Ms_Total_Gpu_Timestamp_Offset * 4 + 4);
            }
            else if (adapter->chip_id == CHIP_CHX004)
            {
                time_stamp_lo = zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS  + Reg_Csp_Ms_Total_Gpu_Timestamp_Offset_CHX004 * 4);
                time_stamp_hi = zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS  + Reg_Csp_Ms_Total_Gpu_Timestamp_Offset_CHX004 * 4 + 4);
            }

            *info->value64 = time_stamp_lo | (time_stamp_hi << 32);
#else
            if (adapter->chip_id == CHIP_ELT3K)
            {
                info->value64 = zx_read64(adapter->mmio + MMIO_CSP_START_ADDRESS  + Reg_Csp_Ms_Total_Gpu_Timestamp_Offset * 4);
            }
            else if (adapter->chip_id == CHIP_CHX004)
            {
                info->value64 = zx_read64(adapter->mmio + MMIO_CSP_START_ADDRESS  + Reg_Csp_Ms_Total_Gpu_Timestamp_Offset_CHX004 * 4);
            }
#endif
           break;

        case ZX_QUERY_REGISTER_U32:
            buf=zx_calloc(info->buf_len*sizeof(unsigned int));
            if(buf)
            {
                for(i=0;i<info->buf_len;i++)
                {
                    *(buf+i)=zx_read32(adapter->mmio+info->argu+i*sizeof(unsigned int));
                }
            }
            zx_copy_to_user(info->buf, buf, info->buf_len*sizeof(unsigned int));
	        zx_free(buf);
            break;

        case ZX_SET_REGISTER_U32:
            zx_write32(adapter->mmio+info->argu, info->value);
            break;
        default:
            break;
    }

    if (adapter->pm_caps.pll_control)
    {
        zx_spin_unlock_irqrestore(adapter->schedule->pll_lock, flags);
    }

}


void vidsch_get_timestamp_e3k(adapter_t *adapter, unsigned long long *time_stamp)
{
    if (adapter->chip_id == CHIP_ELT3K)
    {
        *time_stamp = zx_read64(adapter->mmio + MMIO_CSP_START_ADDRESS  + Reg_Csp_Ms_Total_Gpu_Timestamp_Offset * 4);
    }
    else if (adapter->chip_id == CHIP_CHX004)
    {
        *time_stamp = zx_read64(adapter->mmio + MMIO_CSP_START_ADDRESS  + Reg_Csp_Ref_Total_Gpu_Timestamp_Offset * 4);
    }
}

int vidsch_dump_power_regs_e3k(adapter_t *adapter)
{
    int pwm_regs[PWM_REGS_COUNT];
    int i;

    if(!adapter->pm_caps.pwm_mode) return 0;

    for(i = 0; i < PWM_REGS_COUNT; i++)
    {
       pwm_regs[i] = zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS + (0x11c + i)*4);
    }

    for(i = 0; i < 3; i++)
    {
        zx_info("power regs mmio 0x%08x: 0x%08x 0x%08x 0x%08x 0x%08x 0x%08x \n",
            MMIO_CSP_START_ADDRESS + (0x11c + i*5)*4,
            pwm_regs[i*5], pwm_regs[i*5+1], pwm_regs[i*5+2], pwm_regs[i*5+3], pwm_regs[i*5+4]);
    }

    return 0;
}


static void vidsch_dump_info_e3k(struct os_seq_file *seq_file, adapter_t *adapter)
{
#define GPC_COUNT 3
    EngineSatus_e3k     engine_status = {0};
    unsigned int        *status =(unsigned int*) &engine_status;
    int i;

    vidsch_dump_power_regs_e3k(adapter);

    for( i = 0; i <7; i++)
    {
        *(status + i) = zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS + (Reg_Block_Busy_Bits_Top_Offset + i)*4);
    }
    //print the engine status
    zx_seq_printf(seq_file, "E3K engine status:\n");
    zx_seq_printf(seq_file,"Top: 0x%x \n", engine_status.Top.uint);
    zx_seq_printf(seq_file,"\t Csp_Busy    :%d \n", engine_status.Top.reg.Csp_Busy);
    zx_seq_printf(seq_file,"\t Mxua_Busy   :%d \n", engine_status.Top.reg.Mxua_Busy);
    zx_seq_printf(seq_file,"\t Mxub_Busy   :%d \n", engine_status.Top.reg.Mxub_Busy);
    zx_seq_printf(seq_file,"\t L2_Busy     :%d \n", engine_status.Top.reg.L2_Busy);
    zx_seq_printf(seq_file,"\t Gpcpfe_Busy    :%d \n", engine_status.Top.reg.Gpcpfe_Busy);
    zx_seq_printf(seq_file,"\t Gpcpbe_Busy   :%d \n", engine_status.Top.reg.Gpcpbe_Busy);
    zx_seq_printf(seq_file,"\t Sg_Busy   :%d \n", engine_status.Top.reg.Sg_Busy);
    zx_seq_printf(seq_file,"\t Tasfe_Busy     :%d \n", engine_status.Top.reg.Tasfe_Busy);
    zx_seq_printf(seq_file,"\t Tasbe_Busy   :%d \n", engine_status.Top.reg.Tasbe_Busy);
    zx_seq_printf(seq_file,"\t Hub_Busy     :%d \n", engine_status.Top.reg.Hub_Busy);

    for (i = 0; i < GPC_COUNT; i++)
    {
        Reg_Block_Busy_Bits_Gpc0_0 *s0 = (Reg_Block_Busy_Bits_Gpc0_0 *)(status + i * 2 + 1);
        Reg_Block_Busy_Bits_Gpc0_1 *s1 = (Reg_Block_Busy_Bits_Gpc0_1 *)(status + i * 2 + 2);

        zx_seq_printf(seq_file,"GPC : %d  HEX: 0x%x   0x%x\n",i, s0->uint, s1->uint);
        zx_seq_printf(seq_file,"\t Tgz_Busy    :0x%x \n", s0->reg.Tgz_Busy);
        zx_seq_printf(seq_file,"\t Iu_Busy    :0x%x \n", s0->reg.Iu_Busy);
        zx_seq_printf(seq_file,"\t Wbu_Busy    :0x%x \n", s0->reg.Wbu_Busy);
        zx_seq_printf(seq_file,"\t Wls_Busy    :0x%x \n", s0->reg.Wls_Busy);
        zx_seq_printf(seq_file,"\t Ffc_Busy    :0x%x \n", s0->reg.Ffc_Busy);
        zx_seq_printf(seq_file,"\t Tu_Busy    :0x%x \n", s0->reg.Tu_Busy);

        zx_seq_printf(seq_file,"\t Eu_Constructor_Busy    :0x%x \n", s1->reg.Eu_Constructor_Busy);
        zx_seq_printf(seq_file,"\t Euvs_Busy    :0x%x \n", s1->reg.Euvs_Busy);
        zx_seq_printf(seq_file,"\t Euhs_Busy    :0x%x \n", s1->reg.Euhs_Busy);
        zx_seq_printf(seq_file,"\t Eufe_Busy    :0x%x \n", s1->reg.Eufe_Busy);
        zx_seq_printf(seq_file,"\t Euds_Busy    :0x%x \n", s1->reg.Euds_Busy);
        zx_seq_printf(seq_file,"\t Eugs_Busy    :0x%x \n", s1->reg.Eugs_Busy);
        zx_seq_printf(seq_file,"\t Eups_Busy    :0x%x \n", s1->reg.Eups_Busy);
        zx_seq_printf(seq_file,"\t Eucs_Busy    :0x%x \n", s1->reg.Eucs_Busy);
    }
}

static void vidsch_dump_debugbus_e3k(struct os_printer *p, adapter_t *adapter)
{
    unsigned int     engine_status_offset     = MMIO_CSP_START_ADDRESS + Reg_Block_Busy_Bits_Top_Offset*4;
    EngineSatus_e3k  engine_status;
    unsigned int     *status = (unsigned int *)&engine_status;
    int i;

    for( i = 0; i <7; i++)
    {
        *(status + i) = zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS + (Reg_Block_Busy_Bits_Top_Offset + i)*4);
    }

    zx_printf(p, "ChipSliceMask: %08x\n", adapter->hw_caps.chip_slice_mask);
    zx_printf(p, "Engine Hang and Status: %08x,%08x,%08x,%08x,%08x,%08x,%08x.\n",
        engine_status.Top.uint, engine_status.Gpc0_0.uint, engine_status.Gpc0_1.uint,
        engine_status.Gpc1_0.uint, engine_status.Gpc1_1.uint, engine_status.Gpc2_0.uint, engine_status.Gpc2_1.uint);

    if (adapter->pm_caps.pwm_mode)
    {
        zx_printf(p, "Power Status: %08x\n",
            zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Pwr_Mgr_Status1_Offset * 4));
    }
    vidsch_display_debugbus_info_e3k(adapter, p, FALSE);
}

static vidsch_chip_func_t engine_gfx_low_chip_func_e3k =
{
    .initialize         = engine_gfx_low_init_e3k,
    .destroy            = engine_destroy_e3k,

    .submit             = engine_submit_task_dma_e3k,
    .update_fence_id    = engine_update_fence_id_e3k,
#ifdef ZX_HW_NULL
    .set_fence_id       = engine_set_fence_id_e3k,
#endif

    .cil2_misc          = engine_cil2_misc_e3k,
    .dump_cpll          = engine_dump_cpll_e3k,

    .save               = engine_save_e3k,
    .restore            = engine_gfx_low_restore_e3k,

    .render             = vidsch_render_e3k,
    .patch              = vidsch_patch_e3k,

    .reset_hw           = vidsch_reset_hw_e3k,
    .dump_hang_info     = vidsch_dump_hang_info_e3k,
    .dump_hang           = vidsch_dump_hang_e3k,

    .get_timestamp      = vidsch_get_timestamp_e3k,
    .power_clock        = vidsch_power_clock_on_off_e3k,

};

static vidsch_chip_func_t engine_gfx_high_chip_func_e3k =
{
    .initialize         = engine_gfx_high_init_e3k,
    .destroy            = engine_destroy_e3k,

    .submit             = engine_submit_task_dma_e3k,
    .update_fence_id    = engine_update_fence_id_e3k,

    .cil2_misc        = engine_cil2_misc_e3k,

    .save               = engine_save_e3k,
    .restore            = engine_gfx_high_restore_e3k,

    .render             = vidsch_render_e3k,
    .patch              = vidsch_patch_e3k,

    .reset_hw           = vidsch_reset_hw_e3k,
    .dump_hang_info     = vidsch_dump_hang_info_e3k,
    .dump_hang           = vidsch_dump_hang_e3k,

    .power_clock        = vidsch_power_clock_on_off_e3k,
};

static vidsch_chip_func_t engine_cs_high_chip_func_e3k =
{
    .initialize         = engine_cs_init_e3k,
    .destroy            = engine_destroy_e3k,

    .submit             = engine_submit_task_dma_e3k,
    .update_fence_id    = engine_update_fence_id_e3k,

    .cil2_misc        = engine_cil2_misc_e3k,

    .save               = engine_save_e3k,
    .restore            = engine_restore_e3k,

    .render             = vidsch_render_e3k,
    .patch              = vidsch_patch_e3k,

    .reset_hw           = vidsch_reset_hw_e3k,

    .power_clock   = NULL,
};

static vidsch_chip_func_t engine_cs_low0_chip_func_e3k =
{
    .initialize         = engine_cs_init_e3k,
    .destroy            = engine_destroy_e3k,

    .submit             = engine_submit_task_dma_e3k,
    .update_fence_id    = engine_update_fence_id_e3k,

    .cil2_misc        = engine_cil2_misc_e3k,

    .save               = engine_save_e3k,
    .restore            = engine_restore_e3k,

    .render             = vidsch_render_e3k,
    .patch              = vidsch_patch_e3k,

    .reset_hw           = vidsch_reset_hw_e3k,

    .power_clock   = NULL,
};

static vidsch_chip_func_t engine_cs_low1_chip_func_e3k = 
{
    .initialize         = engine_cs_init_e3k,
    .destroy            = engine_destroy_e3k,

    .submit             = engine_submit_task_dma_e3k,
    .update_fence_id    = engine_update_fence_id_e3k,

    .cil2_misc        = engine_cil2_misc_e3k,

    .save               = engine_save_e3k,
    .restore            = engine_restore_e3k,

    .render             = vidsch_render_e3k,
    .patch              = vidsch_patch_e3k,

    .reset_hw           = vidsch_reset_hw_e3k,

    .power_clock   = NULL,
};

static vidsch_chip_func_t engine_cs_low2_chip_func_e3k =
{
    .initialize         = engine_cs_init_e3k,
    .destroy            = engine_destroy_e3k,

    .submit             = engine_submit_task_dma_e3k,
    .update_fence_id    = engine_update_fence_id_e3k,

    .cil2_misc        = engine_cil2_misc_e3k,

    .save               = engine_save_e3k,
    .restore            = engine_restore_e3k,

    .render             = vidsch_render_e3k,
    .patch              = vidsch_patch_e3k,

    .reset_hw           = vidsch_reset_hw_e3k,
};

static vidsch_chip_func_t engine_cs_low3_chip_func_e3k =
{
    .initialize         = engine_cs_init_e3k,
    .destroy            = engine_destroy_e3k,

    .submit             = engine_submit_task_dma_e3k,
    .update_fence_id    = engine_update_fence_id_e3k,

    .cil2_misc        = engine_cil2_misc_e3k,

    .save               = engine_save_e3k,
    .restore            = engine_restore_e3k,

    .render             = vidsch_render_e3k,
    .patch              = vidsch_patch_e3k,

    .reset_hw           = vidsch_reset_hw_e3k,

    .power_clock   = NULL,
};

static vidsch_chip_func_t engine_vcp_low0_chip_func_e3k =
{
    .initialize         = engine_vcp_init_e3k,
    .destroy            = engine_destroy_e3k,

    .submit             = engine_submit_task_dma_e3k,
    .update_fence_id    = engine_update_fence_id_e3k,

    .cil2_misc        = engine_cil2_misc_e3k,

    .save               = engine_save_e3k,
    .restore            = engine_restore_e3k,

    .render             = vidsch_render_e3k,
    .patch              = vidsch_patch_e3k,

    .reset_hw           = vidsch_reset_hw_e3k,
    .dump_hang_info     = vidsch_dump_hang_info_e3k,
    .dump_hang           = vidsch_dump_hang_e3k,

    .power_clock        = vidsch_power_clock_on_off_e3k,
};

static vidsch_chip_func_t engine_vcp_low1_chip_func_e3k =
{
    .initialize         = engine_vcp_init_e3k,
    .destroy            = engine_destroy_e3k,

    .submit             = engine_submit_task_dma_e3k,
    .update_fence_id    = engine_update_fence_id_e3k,

    .cil2_misc        = engine_cil2_misc_e3k,

    .save               = engine_save_e3k,
    .restore            = engine_restore_e3k,

    .render             = vidsch_render_e3k,
    .patch              = vidsch_patch_e3k,

    .reset_hw           = vidsch_reset_hw_e3k,
    .dump_hang_info     = vidsch_dump_hang_info_e3k,
    .dump_hang           = vidsch_dump_hang_e3k,

    .power_clock        = vidsch_power_clock_on_off_e3k,
};

static vidsch_chip_func_t engine_vpp_chip_func_e3k =
{
    .initialize         = engine_vpp_init_e3k,
    .destroy            = engine_destroy_e3k,

    .submit             = engine_submit_task_dma_e3k,
    .update_fence_id    = engine_update_fence_id_e3k,

    .cil2_misc        = engine_cil2_misc_e3k,

    .save               = engine_save_e3k,
    .restore            = engine_restore_e3k,

    .render             = vidsch_render_e3k,
    .patch              = vidsch_patch_e3k,

    .reset_hw           = vidsch_reset_hw_e3k,
    .dump_hang_info     = vidsch_dump_hang_info_e3k,
    .dump_hang           = vidsch_dump_hang_e3k,

    .power_clock        = vidsch_power_clock_on_off_e3k,
};

vidschedule_chip_func_t vidschedule_chip_func =
{
    .set_dvfs              = NULL,
    .dvfs_tuning           = vidsch_dfs_tuning_e3k,
    .dump_info             = vidsch_dump_info_e3k,
    .dump_debugbus         = vidsch_dump_debugbus_e3k,
    .power_tuning          = vidsch_power_tuning_e3k,
    .get_set_reg           = vidsch_get_set_reg_e3k,
    .disable_dvfs          = vidsch_disable_dvfs_004,
    .pll_tuning            = vidsch_pll_tuning_e3k,
    .pll_on_off            = vidsch_pll_on_off_e3k,
};

