#include "zx_adapter.h"
#include "global.h"
#include "kernel_interface.h"

void glb_init_chip_id(adapter_t *adapter, krnl_adapter_init_info_t *info)
{
    adapter->hw_caps.support_snooping = TRUE;
    adapter->pm_caps.pwm_mode = info->zx_pwm_mode;
    adapter->pm_caps.pll_control = info->zx_pll_control;

    {
        adapter->family_id = FAMILY_ELT;
        adapter->generic_id= PCI_ID_GENERIC_ELITE3K;
        adapter->chip_id = CHIP_ELT3K;

        if((adapter->bus_config.device_id & CHIP_MASK) == CHIP_EXCALIBUR2_CHX004)
        {
            adapter->chip_id= CHIP_CHX004;
        }

        adapter->hw_caps.support_snooping = TRUE;
        //from loongson, loonson 3A3000 not support snoop
#if defined(__mips64__)
        adapter->hw_caps.support_snooping = FALSE;
#endif
        adapter->hw_caps.page_4k_enable   = TRUE;

        adapter->pm_caps.pwm_manual       = FALSE;

        //for elite3000, only cg manual mode is supported.
        //zx_pwm_mode == 1 means enable cg manual mode.
        if (info->zx_pwm_mode)
        {
            adapter->pwm_level.EnableClockGating = 1;
            adapter->pwm_level.EnablePowerGating = 1;
        }

        if (info->zx_pwm_mode)
        {
            unsigned int pwm_mode = info->zx_pwm_mode;
            unsigned int pwm_manual_mode = (pwm_mode >> 4) & 0xF;
            unsigned int pwm_auto_mode = pwm_mode & 0xF;

            zx_assert(((pwm_auto_mode & pwm_manual_mode) == 0), "set zx_pwm_mode error!\n");

#define PWM_3D_MODE 0x1
#define PWM_VCP_MODE 0x2
#define PWM_VPP_MODE 0x4
#define ASSIGN_PWM_MODE(type1, name, type2)\
        do { \
           adapter->pm_caps.type1##_cg_##type2 = pwm_##type2##_mode & PWM_##name##_MODE ? 1:0;\
           adapter->pm_caps.type1##_pg_##type2 = pwm_##type2##_mode & PWM_##name##_MODE ? 1:0;\
        }while(0)

            ASSIGN_PWM_MODE(gfx, 3D, manual);
            ASSIGN_PWM_MODE(vcp, VCP, manual);
            ASSIGN_PWM_MODE(vpp, VPP, manual);

            ASSIGN_PWM_MODE(gfx, 3D, auto);
            ASSIGN_PWM_MODE(vcp, VCP, auto);
            ASSIGN_PWM_MODE(vpp, VPP, auto);
        }

        if(info->zx_dvfs_mode)
        {
            adapter->hw_caps.dvfs_enable    =  TRUE;

            adapter->pm_caps.gfx_dvfs_auto = info->zx_dvfs_mode & 0x1 ? 1:0;
            adapter->pm_caps.vcp_dvfs_auto = info->zx_dvfs_mode & 0x2 ? 1:0;
            adapter->pm_caps.vpp_dvfs_auto = info->zx_dvfs_mode & 0x4 ? 1:0;
            adapter->pm_caps.gfx_mem_dvfs  = info->zx_dvfs_mode & 0x8 ? 1:0;
            adapter->pm_caps.vcp_mem_dvfs  = info->zx_dvfs_mode & 0x10 ? 1:0;
            adapter->pm_caps.vpp_mem_dvfs  = info->zx_dvfs_mode & 0x20 ? 1:0;
            adapter->pm_caps.gfx_output_mode = info->zx_dvfs_mode & 0xc0 ? 1:0;
            adapter->pm_caps.vcp_output_mode = info->zx_dvfs_mode & 0x300 ? 1:0;
            adapter->pm_caps.vpp_output_mode = info->zx_dvfs_mode & 0xc00 ? 1:0;
        }


        adapter->hw_caps.hw_patch_enable  = FALSE;
#ifndef ZX_HW_NULL
        adapter->hw_caps.local_only       = FALSE;
#else
        adapter->hw_caps.local_only       = FALSE;
#endif

#ifdef VIDEO_ONLY_FPGA
        adapter->hw_caps.video_only        = TRUE;
#else
        adapter->hw_caps.video_only        = FALSE;
#endif

#ifdef GFX_ONLY_FPGA
        adapter->hw_caps.gfx_only          = TRUE;
#else
        adapter->hw_caps.gfx_only          = FALSE;
#endif

        if (adapter->hw_caps.video_only || adapter->hw_caps.local_only)
        {
            adapter->ctl_flags.paging_enable = FALSE;
        }
        else
        {
            adapter->ctl_flags.paging_enable        = TRUE;
        }

        adapter->ctl_flags.worker_thread_enable = info->zx_worker_thread_enable;

        adapter->ctl_flags.split_enable         = TRUE;
        adapter->ctl_flags.rename_enable        = TRUE;

        adapter->ctl_flags.vesa_tempbuffer_enable = info->zx_vesa_tempbuffer_enable ? TRUE : FALSE;

        adapter->hw_patch_mask0 |= PATCH_FENCE_INTERRUPT_LOST;
    }

    adapter->ctl_flags.recovery_enable = info->zx_recovery_enable;
    adapter->ctl_flags.run_on_qt = info->zx_run_on_qt;
    adapter->ctl_flags.hang_dump = info->zx_hang_dump;

    adapter->Real_vram_size = ((unsigned long long)info->zx_local_size_g << 30) + ((unsigned long long)info->zx_local_size_m << 20);
    adapter->gart_ram_size = ((unsigned long long)info->zx_pcie_size_g << 30) + ((unsigned long long)info->zx_pcie_size_m << 20);

    if(info->zx_hang_dump)
    {
        adapter->ctl_flags.recovery_enable      = TRUE;
        adapter->ctl_flags.flag_buffer_verify   = info->zx_flag_buffer_verify;

        // if duplicate hang, then will disable paging
        // consider: if pcie_mem=1g, then unsnoop segment will be 265M (since unsnoop is 1/4 of total)
        // then there is no more size of the left.
        if (info->zx_hang_dump == 3)
        {
           adapter->ctl_flags.paging_enable = 0;
        }

        zx_info("hang dump avaiable, set used vram size: %d M, visible vram size: %d M,gart_ram_size:%d M\n", adapter->Real_vram_size >> 20, adapter->Visible_vram_size >> 20, adapter->gart_ram_size >> 20);
        //zx_assert(adapter->Real_vram_size >= adapter->Visible_vram_size, "cur vram size is smaller than visiable vram size.");
    }

    adapter->hw_caps.secure_range_enable       = FALSE;

    adapter->hw_caps.miu_channel_size   = info->miu_channel_size;
    adapter->hw_caps.chip_slice_mask    = info->chip_slice_mask;

#if defined(ZX_VMI_MODE)
    adapter->hw_caps.miu_channel_num    = 1;
#endif

    adapter->hw_caps.nonsnoop_patch_enable = (info->nonsnoop_patch == 1) ? TRUE : FALSE;

    //zx_info("adapter->ctl_flags.worker_thread_enable %x\n", adapter->ctl_flags.worker_thread_enable);
    //zx_info("hw caps: secure:%d, snoop:%d, 4K_page:%d, Paging:%d, local_only:%d, CG:%d, DFS:%d.\n",
    //    adapter->hw_caps.secure_range_enable, adapter->hw_caps.support_snooping, adapter->hw_caps.page_4k_enable,
    //    adapter->ctl_flags.paging_enable,,adapter->hw_caps.local_only, adapter->pwm_level.EnableClockGating, adapter->hw_caps.dfs_enable);

    glb_init_chip_interface(adapter);
}

void glb_get_pci_config_info(adapter_t *adapter)
{
    unsigned long long mmio;
    unsigned long long fb;
    unsigned long  claim_fb_size;
    unsigned short link_status;
    bus_config_t*  bus_config = &adapter->bus_config;
    zx_map_argu_t map    = {0};
    unsigned char  cache_type  = 0;
    alloc_pages_flags_t alloc_flags = {0};

    zx_get_bus_config(adapter->os_device.pdev, bus_config);

    adapter->sys_caps.secure_on = bus_config->secure_on;

    claim_fb_size = bus_config->mem_end_addr[0] - bus_config->mem_start_addr[0] + 1;

    //zx_info("bus claimed cpu access-able vram size: 0x%x, %dM, secure_on: %d\n",
    //    claim_fb_size, claim_fb_size >> 20, adapter->sys_caps.secure_on);

#if defined(__i386__) || defined(__x86_64__) || defined(__mips64__)
    adapter->primary = (bus_config->command & 0x01) ? 1 : 0;
#else
    adapter->primary = 0;
#endif

    adapter->link_width = (bus_config->link_status>>4) & 0x1F;

    mmio = bus_config->reg_start_addr[0];
    fb   = bus_config->mem_start_addr[0] & (~0xFF);/*The base address not right under QNX*/

#if defined(ZX_VMI_MODE)
    adapter->mmio     = 0; //mmio is set 0, because cmodel knows virt_addr of mmio
    adapter->bci_base = (unsigned int*)(adapter->mmio + 0x10000);

    adapter->vidmm_bus_addr = fb;

    adapter->physical_bus_base_length[0] = bus_config->reg_end_addr[0] - bus_config->reg_start_addr[0];
    adapter->physical_bus_base_length[1] = claim_fb_size;
    adapter->Visible_vram_size = adapter->physical_bus_base_length[1];

    //Real_vram_size is transferred to core, by mem_start_addr[1]/mem_end_addr[1]
    //adapter->Real_vram_size    = bus_config->mem_end_addr[1] - bus_config->mem_start_addr[1] + 1;

    zx_info("video memory size: %dM.\n", adapter->Visible_vram_size >> 20);
#elif defined(ZX_PCIE_BUS)
    map.flags.cache_type = ZX_MEM_UNCACHED;
    map.flags.mem_space  = ZX_MEM_KERNEL;
    map.flags.mem_type   = ZX_SYSTEM_IO;
    map.phys_addr        = mmio;
    map.size             = bus_config->reg_end_addr[0] - bus_config->reg_start_addr[0];

    adapter->mmio_vma = zx_map_io_memory(NULL, &map);

    adapter->mmio     = adapter->mmio_vma->virt_addr;
    adapter->bci_base = (unsigned int*)(adapter->mmio + 0x10000);

    adapter->vidmm_bus_addr = fb;

    adapter->physical_bus_base_length[0] = bus_config->reg_end_addr[0] - bus_config->reg_start_addr[0];
    adapter->physical_bus_base_length[1] = claim_fb_size;

    /*adapter->Real_vram_size will setup later by update adapter info*/
    adapter->Visible_vram_size = adapter->physical_bus_base_length[1];
#elif defined(ZX_HW_NULL)
    //cache_type = ZX_MEM_UNCACHED;
    //alloc_flags.cache_type = cache_type;

    alloc_flags.need_flush = 1;
    alloc_flags.need_zero  = 1;
    alloc_flags.fixed_page = 1;

    map.memory = zx_allocate_pages_memory(0x80000, adapter->os_page_size, alloc_flags);

    if(map.memory == NULL)
    {
        zx_error("allocate command buffer fail: out of memory!\n");
    }

    map.flags.mem_space  = ZX_MEM_KERNEL;
    map.flags.mem_type   = ZX_SYSTEM_RAM;
    map.size             = 0x80000;

    adapter->mmio_vma = zx_map_pages_memory(NULL, &map);
    adapter->mmio_mem = map.memory;

    adapter->mmio     = adapter->mmio_vma->virt_addr;
    adapter->bci_base = (unsigned int*)(adapter->mmio + 0x10000);
    adapter->vidmm_bus_addr = fb;

    adapter->pmu_mmio_vma = zx_map_pages_memory(NULL, &map);
    adapter->pmu_mmio     = adapter->pmu_mmio_vma->virt_addr;

    adapter->physical_bus_base_length[0] = 0x80000;
    adapter->physical_bus_base_length[1] = claim_fb_size;

    adapter->Visible_vram_size     =
    adapter->Real_vram_size                = claim_fb_size;

    zx_info("video memory size: %dM.\n", claim_fb_size >> 20);
    zx_info("mmio virt addr: %x.\n", adapter->mmio);

#else
    //engine mmio
    map.flags.cache_type = ZX_MEM_UNCACHED;
    map.flags.mem_space  = ZX_MEM_KERNEL;
    map.flags.mem_type   = ZX_SYSTEM_IO;
    map.phys_addr        = mmio;
    map.size             = bus_config->reg_end_addr[0] - bus_config->reg_start_addr[0];

    adapter->mmio_vma    = zx_map_io_memory(NULL, &map);
    adapter->mmio        = adapter->mmio_vma->virt_addr;
    adapter->bci_base    = (unsigned int*)(adapter->mmio + 0x10000);
    adapter->vidmm_bus_addr = fb;

    //pmu mmio
    map.phys_addr        = bus_config->reg_start_addr[3];
    map.size             = bus_config->reg_end_addr[3] - bus_config->reg_start_addr[3];

    adapter->pmu_mmio_vma = zx_map_io_memory(NULL, &map);
    adapter->pmu_mmio     = adapter->pmu_mmio_vma->virt_addr;
    adapter->pmu_phys_addr= bus_config->reg_start_addr[3];

    //gpio
    map.phys_addr        = bus_config->reg_start_addr[2];
    map.size             = bus_config->reg_end_addr[2] - bus_config->reg_start_addr[2];

    adapter->gpio_vma    = zx_map_io_memory(NULL, &map);
    adapter->gpio        = adapter->gpio_vma->virt_addr;

    //pmc
    map.phys_addr        = bus_config->reg_start_addr[1];
    map.size             = bus_config->reg_end_addr[1] - bus_config->reg_start_addr[1];

    adapter->pmc_vma      = zx_map_io_memory(NULL, &map);
    adapter->pmc          = adapter->pmc_vma->virt_addr;
    adapter->pmc_phys_addr= bus_config->reg_start_addr[1];

    adapter->physical_bus_base_length[0] = 0x80000;
    adapter->physical_bus_base_length[1] = claim_fb_size;

    zx_info("sys_secure_on: %d, mmio: %x-%d-%p, pmu %x-%d-%p, gpio %x-%d-%p, pmc %x-%d-%p\n",
        bus_config->secure_on,
        bus_config->reg_end_addr[0], bus_config->reg_end_addr[0] - bus_config->reg_start_addr[0], adapter->mmio,
        bus_config->reg_end_addr[3], bus_config->reg_end_addr[3] - bus_config->reg_start_addr[3], adapter->pmu_mmio,
        bus_config->reg_end_addr[2], bus_config->reg_end_addr[2] - bus_config->reg_start_addr[2], adapter->gpio,
        bus_config->reg_end_addr[1], bus_config->reg_end_addr[1] - bus_config->reg_start_addr[1], adapter->pmc);

#endif
}
void glb_fini_bus_config(adapter_t *adapter)
{
    if(adapter->mmio_vma)
    {
        zx_unmap_io_memory(adapter->mmio_vma);
    }

    if(adapter->pmu_mmio_vma)
    {
        zx_unmap_io_memory(adapter->pmu_mmio_vma);
    }

    if(adapter->pmc_vma)
    {
        zx_unmap_io_memory(adapter->pmc_vma);
    }
}

