#include "zx_adapter.h"
#include "global.h"
#include "vidmm.h"
#include "vidsch.h"
#include "vidschi.h"
#include "vidsch_submit.h"
#include "e3k/vidsch/vidsch_engine_e3k.h"
#include "e3k/include/mm_e3k.h"
#include "vidmm/vidmmi.h"

#define MEM_FILL_PATTERN 0x74369216
#define VALIDATE_MEM_START_ADDR 0x100000000ULL //4G start
//#define VALIDATE_MEM_START_ADDR 0x140000000ULL //for 256M local mem, the right addr start over 4G: 5G
#define VALIDATE_MEM_SIZE 0x1000 //4k size
#define VALIDATE_MEM_PIXEL_SIZE 1024

unsigned int vidsch_get_pte_value(adapter_t *adapter, unsigned long long phy_addr){
    PTE_L3_t page_entry = {0};

    page_entry.pte_4k.Valid = 1;
    page_entry.pte_4k.Segment = 0;
    page_entry.pte_4k.NS = 1;
    page_entry.pte_4k.Snoop = FALSE;

    phy_addr >>= 12;
    if(phy_addr > (adapter->low_top_address >> 12))
        phy_addr -= (0x100000 - (adapter->low_top_address >> 12));

    page_entry.pte_4k.Addr = phy_addr & 0xFFFFFFF;

    return page_entry.uint;
}

//extern void* zx_ioremap_wc(unsigned long io_base, unsigned long size);
void vidsch_test_memory(adapter_t *adapter)
{
    engine_share_e3k_t *share = NULL;
    vidsch_mgr_t *sch_mgr = NULL;
    vidschedule_t *schedule = adapter->schedule;
    dma_node_t *dma_clear = NULL;
    FASTCLEAR_REGSETTING_E3K *pClearCmd0 = NULL;
    FASTCLEAR_REGSETTING_E3K *pClearCmd = NULL;
    int j = 0;
    unsigned int *mem=NULL, *tmpMem=NULL;
    int rightCnt=0, wrongCnt=0;
    task_dma_t task_dma = {0};
    unsigned long long fence_id = 0;
    unsigned int *gart_ptes = adapter->gart_table_L3->virt_addr;
    vidmm_mgr_t *mm_mgr = adapter->mm_mgr;
    list_node_t *list_node = NULL;
    int pte_offset = 0;
    unsigned long long phys_addr = 0;

    share = adapter->private_data;
    sch_mgr = adapter->sch_mgr[adapter->paging_engine_index];
    dma_clear = vidschi_allocate_dma_node(adapter->sch_mgr[adapter->paging_engine_index], 16*4*1024);
    pClearCmd0 = (FASTCLEAR_REGSETTING_E3K*)dma_clear->virt_base_addr;

    list_node = heap_allocate(&mm_mgr->segment[SEGMENT_ID_GART_UNSNOOPABLE_E3K].heap, VALIDATE_MEM_SIZE, adapter->os_page_size, 0);
    if(!list_node){
        zx_info("###Dbg: %s allocation list node failed!\n", __func__);
        return;
    }

    pte_offset = (list_node->aligned_offset - adapter->gart_table_L3->gart_start_addr) / adapter->os_page_size;

    pClearCmd = zx_calloc(sizeof(FASTCLEAR_REGSETTING_E3K));

    zx_memcpy(pClearCmd, &share->fast_clear_cmd_e3k, sizeof(FASTCLEAR_REGSETTING_E3K));

    pClearCmd->DstFormat.reg.Format        = HSF_B8G8R8A8_UNORM;
    pClearCmd->DstFormat.reg.Bl_Slot_Idx    = 0;
    pClearCmd->DstDepth.reg.Depth_Or_Arraysize= 1;
    pClearCmd->DstDepth.reg.Range_Type      = 0;

    pClearCmd->DstViewCtrl.reg_CHX004.Arraysize = 1;
    pClearCmd->DstViewCtrl.reg_CHX004.First_Array_Idx = 0;
    
    pClearCmd->DstMisc.reg.Is_Tiling        = 0; //Always treate it as linear.
    pClearCmd->DstMisc.reg.Rt_Enable        = 1;
    pClearCmd->DstMisc.reg.Rt_Write_Mask    = 0xF;
    pClearCmd->DstMisc.reg.Resource_Type = RT_MISC_RESOURCE_TYPE_2D_TEXTURE;
    pClearCmd->DstMisc.reg.Eu_Blend_Enable  = 0; //Always treate it as linear.

    pClearCmd->Dzs_Ctrl.reg.Src_Read_Alloc_En = 0;

    pClearCmd->Rast_Ctrl.reg.Check_Board = RAST_CTRL_CHECK_BOARD_64X16;
    pClearCmd->Rast_Ctrl.reg.Eub_En = 0;

    pClearCmd->reg_Iu_Ctrl_Ex.reg.Eu_Blending_En = 0;

    pClearCmd->Zs_Req_Ctrl.reg.D_Alloc_En = 1;

    pClearCmd->Fast_Clear_Color0.reg.Color = MEM_FILL_PATTERN;
    pClearCmd->Fast_Clear_Color1.reg.Color = MEM_FILL_PATTERN;
    pClearCmd->Fast_Clear_Color2.reg.Color = MEM_FILL_PATTERN;
    pClearCmd->Fast_Clear_Color3.reg.Color = MEM_FILL_PATTERN;

    pClearCmd->DstSize.reg.Width            = 1024;
    pClearCmd->DstSize.reg.Height           = 1;   
    pClearCmd->RectX.Xmin = 0;
    pClearCmd->RectX.Xmax = 1024 - 1;
    pClearCmd->RectY.Ymin = 0;
    pClearCmd->RectY.Ymax = 1 - 1;

    task_dma.dma_type = PAGING_DMA;
    task_dma.need_hwctx_switch = FALSE;
    task_dma.command_length = sizeof(FASTCLEAR_REGSETTING_E3K);
    task_dma.dma_buffer_node = dma_clear;
    task_dma.submit_start_offset = 0;
    task_dma.submit_end_offset = sizeof(FASTCLEAR_REGSETTING_E3K);

    pClearCmd->DstAddr.reg.Base_Addr = list_node->aligned_offset >> 8;
    zx_memcpy(pClearCmd0, pClearCmd, sizeof(FASTCLEAR_REGSETTING_E3K));
    if(sch_mgr->dma_reserved_memory->vma->need_flush_cache)
    {
        zx_flush_cache(adapter->os_device.pdev, sch_mgr->dma_reserved_memory->vma, sch_mgr->dma_reserved_memory->pages_mem, 0, sch_mgr->dma_reserved_memory->vma->size);
    }

    phys_addr = VALIDATE_MEM_START_ADDR;
    gart_ptes[pte_offset] = vidsch_get_pte_value(adapter, phys_addr);

    /* invalidate the all gart table */
    zx_write32(adapter->mmio + 0x49000 + 0x6*4 , 0);
    zx_write32(adapter->mmio + 0x49000 + 0x7*4,  0);
    zx_write32(adapter->mmio + 0x49000 + 0x8*4, (0x2<<5));

    zx_down_read(schedule->rw_lock);
    /* lock engine */
    zx_mutex_lock(sch_mgr->engine_lock);

    /* enable interrupt just before first submit, so close to the interrupt generated.
    * it because if enable earily, sometime there will be no interrupt generated.
    * don't known why, maybe some VBIOS operation lead this.
    */
    if(sch_mgr->init_submit)
    {
#ifndef ZX_HW_NULL
         zx_enable_interrupt(adapter->os_device.pdev);
#endif
    }

    /* debug use */
    mem = tmpMem = (unsigned int*)zx_ioremap(phys_addr, VALIDATE_MEM_SIZE);
    for(j=0; j<VALIDATE_MEM_PIXEL_SIZE; j++){
        *tmpMem = 0xeeff0011;
        tmpMem++;
    }
    zx_iounmap(mem);

    fence_id = vidschi_inc_send_fence_id(sch_mgr, FALSE);
    task_dma.desc.fence_id     = fence_id;
    /* call chip submit dma buffer */
    sch_mgr->chip_func->submit(adapter, sch_mgr, &task_dma);
    vidsch_wait_fence_back(adapter, adapter->paging_engine_index, fence_id);
    zx_info("###Dbg: after wait fence, page phys addr:0x%llx\n", phys_addr);

    /* unlock engine */
    zx_mutex_unlock(sch_mgr->engine_lock);
    zx_up_read(schedule->rw_lock);

    mem = tmpMem = (unsigned int*)zx_ioremap(phys_addr, VALIDATE_MEM_SIZE);
    for(j=0; j<VALIDATE_MEM_PIXEL_SIZE; j++){
        if(*tmpMem == MEM_FILL_PATTERN){
            zx_info("###cpu read right value:0x%x\n", *tmpMem);
            rightCnt++;
        }else{
            zx_info("@@@cpu read wrong value:0x%x\n", *tmpMem);
            wrongCnt++;
        }
        tmpMem++;
    }
    zx_iounmap(mem);

    zx_info("&&&total int cnt:%d, validate right cnt:%d, wrong cnt:%d\n", VALIDATE_MEM_PIXEL_SIZE, rightCnt, wrongCnt);

    if(pClearCmd)
        zx_free(pClearCmd);

    if(list_node)
        heap_release(&mm_mgr->segment[SEGMENT_ID_GART_UNSNOOPABLE_E3K].heap, list_node);
}
