/*
 * Copyright (c) 2018 Shanghai Zhaoxin Semiconductor Co., Ltd.
 * All Rights Reserved.
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Shanghai Zhaoxin Semiconductor Co., Ltd.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Shanghai Zhaoxin Semiconductor Co., Ltd.
 *
 * The copyright of the source code is protected by the copyright laws of the People's
 * Republic of China and the related laws promulgated by the People's Republic of China
 * and the international covenant(s) ratified by the People's Republic of China.
 *
 */
#ifndef _H_ZX_BUFMGR_H
#define _H_ZX_BUFMGR_H

#ifndef EXTERN_C
#ifdef __cplusplus
#define EXTERN_C extern "C"
#else
#define EXTERN_C extern
#endif
#endif

#include "zx_keinterface.h"
#include "zx_list.h"


#if defined(DEBUG)
#include <syslog.h>
#endif

#ifdef DEBUG
    #define BUFMGR_LOG_TAG                "gpu[bufmgr]:"
    #define bufmgr_info(fmt, args...)     syslog(LOG_INFO, BUFMGR_LOG_TAG fmt, ##args)
    #define bufmgr_dbg(fmt, args...)      syslog(LOG_DEBUG, BUFMGR_LOG_TAG fmt, ##args)
    #define bufmgr_err(fmt, args...)      syslog(LOG_ERR, BUFMGR_LOG_TAG fmt, ##args)
    #define bufmgr_assert(expr)           do {if (!(expr)) { \
                                                syslog(LOG_CRIT, BUFMGR_LOG_TAG "Assert (%s) failed at %s:%d", #expr, __FILE__, __LINE__); exit(-1); \
                                            }}while(0)
#else
    #define bufmgr_info
    #define bufmgr_dbg
    #define bufmgr_err
    #define bufmgr_assert
#endif

typedef struct _zx_bo              zx_bo_t;
typedef struct _zx_bo_alloc_arg    zx_bo_alloc_arg_t;
typedef struct _zx_resource        zx_resource_t;
typedef struct _zx_bufmgr          zx_bufmgr_t;
typedef struct _zx_rename_resource zx_rename_resource_t;

struct _zx_bo_alloc_arg
{
    unsigned int    width;
    unsigned int    height;
    unsigned int    usage_mask;
    zx_format      format;
    zx_access_hint access_hint;
    unsigned int    unpagable      :1;
    unsigned int    tiled          :1;
    unsigned int    primary        :1;
    unsigned int    compressed     :1;
    void *          user_ptr;
    unsigned int    user_buf_size;
};

struct _zx_bo
{
    struct hlist_node link;
    zx_bufmgr_t    *bufmgr;

    unsigned int    refcount;
    unsigned int    mapcount;

    unsigned int    size;
    unsigned int    width;
    unsigned int    height;

    unsigned int    alignment;

    unsigned int    hw_format;
    unsigned int    compress_format;
    unsigned int    bl_slot_index;

    unsigned int    unpagable   :1;
    unsigned int    tiled       :1;
    unsigned int    secured     :1;
    unsigned int    has_pages   :1;
    unsigned int    cpu_visible :1;
    unsigned int    force_clear :1;
    unsigned int    maybe_shared:1;

    unsigned int    bit_cnt;
    unsigned int    pitch;
    unsigned long long    gpu_virt_addr;  // gpu phy address
    unsigned long long    cpu_phy_addr;
    unsigned int    allocation;
    unsigned int    name;
    unsigned long long  gtt_offset;

    unsigned int    sync_obj;
    unsigned long long    fence_addr;

    unsigned int    segment_id;

    unsigned int    use_aperture;
    void            *virt_addr;  // mapped user space address
    int             fourCC;         //for video
    int             invalid_layout;

    struct list_head res_link;
    zx_resource_t  *resource;      // bind to one resource

    struct list_head rename_link;
    zx_rename_resource_t *rename;
    zx_bo_alloc_arg_t alloc_arg;

    unsigned int     __gid;   // should only used for debug
};


typedef union map_flags {
    struct {
        unsigned int acquire_aperture       :1;
        unsigned int read_only              :1;
        unsigned int write_only             :1;
        unsigned int cache_type             :3;
        unsigned int discard                :1;
        unsigned int no_existing_reference  :1;
        unsigned int delay_map              :1;
    };
    unsigned int value;
} map_flags;

typedef struct
{
    map_flags flags;
    void *virt_addr;
    unsigned int allocation;
    unsigned int prefault_num;
} map_args_t;

struct _zx_resource
{
    zx_bufmgr_t        *bufmgr;
    unsigned int        handle;
    struct list_head    link;
    struct list_head    bo_list;
};

struct _zx_rename_resource
{
    unsigned int        max_length;

    unsigned int        allocation_num;

    struct list_head    reference_list;
    struct list_head    unreference_list;
};

#define BO_HASH_SIZE    (32)
#define BO_HASH(handle) ((unsigned int)(handle) % BO_HASH_SIZE)
struct _zx_bufmgr
{
    int fd;
    int enable_lock;
    unsigned int hDevice;

    int refcount;
    struct list_head link;

    pthread_mutex_t mutex;
    unsigned int resource_id;
    struct list_head resource_list;
    struct hlist_head bo_hash[BO_HASH_SIZE];
};

#ifdef __INTERFACE_IMPLEMENT_DEFINED__

EXTERN_C zx_bufmgr_t *zx_bufmgr_init(int fd);
EXTERN_C void zx_bufmgr_destroy(zx_bufmgr_t *bufmgr);

EXTERN_C zx_bo_t *zx_bo_alloc(zx_bufmgr_t *bufmgr, zx_bo_alloc_arg_t *arg);
EXTERN_C zx_bo_t *zx_bo_rename_alloc(zx_bufmgr_t *bufmgr, zx_bo_t *ref);
EXTERN_C zx_bo_t *zx_bo_get_from_handle(zx_bufmgr_t *bufmgr, unsigned int handle);
EXTERN_C zx_bo_t *zx_bo_create_from_handle(zx_bufmgr_t *bufmgr, unsigned int handle, int width, int height, int pitch, int bit_cnt, int format);
EXTERN_C zx_bo_t *zx_bo_create_from_userptr(zx_bufmgr_t *bufmgr, void *userptr, int size, int width, int height, int pitch, int format);
EXTERN_C zx_bo_t *zx_bo_create_from_fd(zx_bufmgr_t *bufmgr, int fd, int width, int height, int pitch, int bit_cnt, int format);
EXTERN_C int zx_bo_relayout(zx_bo_t *bo, int width, int height, int pitch, int bit_cnt, int format);

EXTERN_C void zx_bo_reference(zx_bo_t *bo);
EXTERN_C void zx_bo_unreference(zx_bo_t *bo);
EXTERN_C int zx_bo_is_busy(zx_bo_t *bo);
EXTERN_C int zx_bo_map(zx_bo_t *bo, map_args_t *arg);
EXTERN_C int zx_bo_unmap(zx_bo_t *bo);
EXTERN_C int zx_bo_get_fd(zx_bo_t *bo, int *fd);
EXTERN_C void zx_bo_wait_idle(zx_bo_t *bo);
EXTERN_C int zx_bo_dump_bmp(zx_bo_t *bo, const char *filename);

EXTERN_C unsigned int zx_bo_get_name(zx_bo_t *bo);
EXTERN_C zx_bo_t *zx_bo_create_from_name(zx_bufmgr_t *bufmgr, unsigned int name, int width, int height, int pitch, int bit_cnt, int format);

EXTERN_C int zx_bo_create_resource (
        zx_bufmgr_t           *bufmgr,
        unsigned int           device,
        unsigned int           allocation_num,
        zx_create_alloc_info_t *allocation_info,
        unsigned int           create_resource_flag,
        unsigned int           *resource_handle
);

EXTERN_C int zx_bo_destroy_resource(
        zx_bufmgr_t    *bufmgr,
        unsigned int    device,
        unsigned int    allocation_num,
        unsigned int    *allocation_info,
        unsigned int    resource_handle
);

#else

#define zx_bufmgr_init CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bufmgr_init
#define zx_bufmgr_destroy CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bufmgr_destroy
#define zx_bo_alloc CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bo_alloc
#define zx_bo_rename_alloc CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bo_rename_alloc
#define zx_bo_get_from_handle CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bo_get_from_handle
#define zx_bo_create_from_handle CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bo_create_from_handle
#define zx_bo_create_from_userptr CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bo_create_from_userptr
#define zx_bo_create_from_fd CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bo_create_from_fd
#define zx_bo_relayout CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bo_relayout
#define zx_bo_reference CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bo_reference
#define zx_bo_unreference CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bo_unreference
#define zx_bo_is_busy CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bo_is_busy
#define zx_bo_map CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bo_map
#define zx_bo_unmap CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bo_unmap
#define zx_bo_get_fd CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bo_get_fd
#define zx_bo_wait_idle CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bo_wait_idle
#define zx_bo_dump_bmp CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bo_dump_bmp
#define zx_bo_get_name CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bo_get_name
#define zx_bo_create_from_name CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bo_create_from_name
#define zx_bo_create_resource CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bo_create_resource
#define zx_bo_destroy_resource CONCAT(bufmgr_interface_v2,DRIVER_NAME)->bo_destroy_resource

#endif

struct bufmgr_interface {
    zx_bufmgr_t * (*bufmgr_init) (int fd);
    void  (*bufmgr_destroy) (zx_bufmgr_t *bufmgr);
    zx_bo_t * (*bo_alloc) (zx_bufmgr_t *bufmgr, zx_bo_alloc_arg_t *arg);
    zx_bo_t * (*bo_rename_alloc) (zx_bufmgr_t *bufmgr, zx_bo_t *ref);
    zx_bo_t * (*bo_get_from_handle) (zx_bufmgr_t *bufmgr, unsigned int handle);
    zx_bo_t * (*bo_create_from_handle) (zx_bufmgr_t *bufmgr, unsigned int handle, int width, int height, int pitch, int bit_cnt, int format);
    zx_bo_t * (*bo_create_from_userptr) (zx_bufmgr_t *bufmgr, void *userptr, int size, int width, int height, int pitch, int format);
    zx_bo_t * (*bo_create_from_fd) (zx_bufmgr_t *bufmgr, int fd, int width, int height, int pitch, int bit_cnt, int format);
    int  (*bo_relayout) (zx_bo_t *bo, int width, int height, int pitch, int bit_cnt, int format);
    void  (*bo_reference) (zx_bo_t *bo);
    void  (*bo_unreference) (zx_bo_t *bo);
    int  (*bo_is_busy) (zx_bo_t *bo);
    int  (*bo_map) (zx_bo_t *bo, map_args_t *arg);
    int  (*bo_unmap) (zx_bo_t *bo);
    int  (*bo_get_fd) (zx_bo_t *bo, int *fd);
    void  (*bo_wait_idle) (zx_bo_t *bo);
    int  (*bo_dump_bmp) (zx_bo_t *bo, const char *filename);
    unsigned int  (*bo_get_name) (zx_bo_t *bo);
    zx_bo_t * (*bo_create_from_name) (zx_bufmgr_t *bufmgr, unsigned int name, int width, int height, int pitch, int bit_cnt, int format);

    int (*bo_create_resource) (
        zx_bufmgr_t           *bufmgr,
        unsigned int           device,
        unsigned int           allocation_num,
        zx_create_alloc_info_t *allocation_info,
        unsigned int           create_resource_flag,
        unsigned int           *resource_handle
    );

    int (*bo_destroy_resource)(
        zx_bufmgr_t    *bufmgr,
        unsigned int    device,
        unsigned int    allocation_num,
        unsigned int    *allocation_info,
        unsigned int    resource_handle
    );
};
EXTERN_C struct bufmgr_interface *CONCAT(bufmgr_interface_v2,DRIVER_NAME);

#endif
