/*
 * Copyright (c) 2018 Shanghai Zhaoxin Semiconductor Co., Ltd.
 * All Rights Reserved.
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Shanghai Zhaoxin Semiconductor Co., Ltd.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Shanghai Zhaoxin Semiconductor Co., Ltd.
 *
 * The copyright of the source code is protected by the copyright laws of the People's
 * Republic of China and the related laws promulgated by the People's Republic of China
 * and the international covenant(s) ratified by the People's Republic of China.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "zx_hwinfo.h"
#define DEBUSINFO "/sys/kernel/debug/dri/0/zx/info"
#define DEBUGCOUNTER 0

extern int zx_context_get_current_pid();
extern int zx_context_get_current_tid();
extern unsigned long long zx_context_get_current_system_time();


 int  gfx_get_struct_size(int fd)
 {
    if (fd<0)
    {
        return 0;
    }
    else
    {
        return sizeof(zx_dev_t);
    }
    
 }

 static  DWORD SEND_VCP_QUERY_E3K(DWORD BlockId, DWORD Offset, DWORD DumpSize, large_inter  address, DWORD* pCmd)
{
   
    Vcp_Decouple_Opcodes_cmd         cmd_CSP_Vcp = {0};
    DWORD* pCmd0 = pCmd;
    cmd_CSP_Vcp.cmd_Vcp_Query_Dump.Major_Opcode = VCP_OPCODE_Vcp_Query_Dump;
    cmd_CSP_Vcp.cmd_Vcp_Query_Dump.Block_Id     = BlockId;
    cmd_CSP_Vcp.cmd_Vcp_Query_Dump.Dump_Dw_Cnt  = DumpSize;
    cmd_CSP_Vcp.cmd_Vcp_Query_Dump.Reg_Offset   = Offset;

    *pCmd++ = cmd_CSP_Vcp.uint;
    *pCmd++ = address.low32;
    *pCmd++ = address.high32;

    return (DWORD)(pCmd - pCmd0);
}

static DWORD  SEND_INTERNAL_FENCE(DWORD Block, HWM_SYNC_SLOT_E3K Slot, DWORD Value,DWORD* pCmd)
{
    DWORD * pCmd0 = pCmd;
    Vcp_Decouple_Opcodes_cmd     SendFenceCmd={0};
    Cmd_Vcp_Fence_Internal_Dword1     DW1={0};

    SendFenceCmd.cmd_Vcp_Fence.Major_Opcode        = VCP_OPCODE_Vcp_Fence;//0x8
    SendFenceCmd.cmd_Vcp_Fence.Block_Id            = Block;
    SendFenceCmd.cmd_Vcp_Fence.Slot_Id             = Slot;
    //SendFenceCmd.cmd_Vcp_Fence.Update_Value        = Value;
    SendFenceCmd.cmd_Vcp_Fence.Fence_Type          = VCP_FENCE_FENCE_TYPE_INTERNAL;    
    SendFenceCmd.cmd_Vcp_Fence.Fence_Update_Mode   = VCP_FENCE_FENCE_UPDATE_MODE_COPY;//0x0
    //SendFenceCmd.cmd_Vcp_Fence.Fence_Update_Timing = VCP_INTERNAL_FENCE_FENCE_UPDATE_TIMING_UPDATE_AFTER_WRITE;//0x1
    SendFenceCmd.cmd_Vcp_Fence.Dwc = 1;

    DW1.Update_Value = Value;

    *pCmd++ = (DWORD)SendFenceCmd.uint;
    *pCmd++ = *(DWORD*)(&DW1);

    

    return (DWORD)(pCmd-pCmd0); 
}




int zx_read_reg(int fd,unsigned int offset,unsigned int len,void *buf)
{
    zx_query_info_t info;
    if(buf)
    {
        info.argu=offset;
        info.type=ZX_QUERY_REGISTER_U32;
        info.buf_len=len;
        info.buf=buf;

    }
    return zxQueryInfo(fd,&info);

}
int zx_set_reg(int fd, unsigned int offset,unsigned int value)
{
    zx_query_info_t info;
   
    info.argu=offset;
    info.type=ZX_SET_REGISTER_U32;
    info.value=value;
    //printf("value=%x\n",value);

    return zxQueryInfo(fd,&info);
}

int zx_set_reg_mask(int fd, unsigned int offset, unsigned int value, unsigned int mask)
{
    
    zx_query_info_t info;
    unsigned int org_val   =0;
    unsigned int write_val =0;
  
    zx_read_reg(fd, offset, 1, &org_val);

    write_val=(org_val & (~mask)) | (value & mask);

    return zx_set_reg(fd, offset, write_val);

}
void read_mxu1_counter(int fd, unsigned int *buf)
{
 
   //enable  mxu1 counter  update per lucas zhou
   zx_set_reg_mask(fd, MXU_DIU_RD_ENABLE_OFFSET, 0x80000000, MXU_DIU_RE_ENABLE_MASK);
   zx_set_reg_mask(fd, MXU_DIU_RD_ENABLE_OFFSET, 0x00000000, MXU_DIU_RE_ENABLE_MASK);
   usleep(5);

   zx_read_reg(fd, MXU_DIU_RD_NUM_OFFSET_E3K, MXU1_COUNTER_SZIE, (void *)buf);
}

int gfx_get_usage_info(int fd,void *pdev, gf_hwq_info *pengineinfo)
{
    zx_query_info_t info;
    int ret=0;
    if(pengineinfo)
    {
        info.type=ZX_QUERY_ENGINE_USAGE_STATUS;
        info.buf_len=sizeof(pengineinfo);
        info.buf=pengineinfo;
        ret=zxQueryInfo(fd,&info);
    }
        return ret;
}

int gfx_get_usage_info_ext(int fd,void *pdev, gfx_hwq_info_ext *pengineinfo)
{
    zx_query_info_t info;
    int ret=0;
    if(pengineinfo)
    {
        info.type=ZX_QUERY_ENGINE_USAGE_STATUS_EXT;
        info.buf_len=sizeof(pengineinfo);
        info.buf=pengineinfo;
        ret=zxQueryInfo(fd,&info);
    }
        return ret;
}

int gfx_init_hwinfo(int fd, void *pdev)
{
    zx_create_device_t createDevice = { 0 };
    zx_bo_alloc_arg_t boAllocArg = { 0 };
    zx_dev_t *dev=(zx_dev_t*)pdev;

    Reg_Csp_Misc_Control  reg_Csp_Misc_Control      ={0};
    int ret=0;

    memset(dev,0,sizeof(zx_dev_t));  
    if(fd<0)
    {
           
        return -1;

    }

    dev->zxFd=fd;
    ret=zxCreateDevice(dev->zxFd, &createDevice);
    dev->zxDevice = createDevice.device;
    dev->bufmgr = zx_bufmgr_init(dev->zxFd);
    dev->renderContext = zx_create_context(dev->zxFd, dev->zxDevice, RB_INDEX_VCP1);
    if (dev->bo==0) 
    {
        boAllocArg.width = 4096;
        boAllocArg.height = 1;
        boAllocArg.tiled = 0;
        boAllocArg.format = ZX_FORMAT_UINT32;
        boAllocArg.usage_mask = ZX_USAGE_TEMP_BUFFER;
        boAllocArg.access_hint = ZX_ACCESS_GPU_ALMOST;
        boAllocArg.unpagable = 0;
        boAllocArg.tiled= 0;
        dev->bo = zx_bo_alloc(dev->bufmgr, &boAllocArg);          
    
    }

    dev->counter.hCounterAlloc=dev->bo->allocation;
    dev->counter.gpu_virt_addr=dev->bo->gpu_virt_addr;
    dev->counter.mode=1;
    dev->counter.CounterSize=HWINFO_COUTER_SIZE;

    zx_read_reg(fd,MMIO_Csp_Misc_Control_Offset,1,&reg_Csp_Misc_Control);
    reg_Csp_Misc_Control.reg.Ila_Mode=dev->counter.mode & 0xf;
    zx_set_reg(fd,MMIO_Csp_Misc_Control_Offset,reg_Csp_Misc_Control.uint);

    gfx_get_hwinfo (fd,pdev, NULL);

    return ret;
}


int  gfx_enable_dbevent(int fd, void *pdev)
{
    zx_dev_t *dev=(zx_dev_t*)pdev;
    int ret=0;

    if(fd<0)
    {
        return -1;
    }
    dev->addperfevent=1;

    return ret;
}

int  gfx_get_hwinfo (int fd,void *pdev, zx_meminfo *pmeminfo)
{
    
    DWORD* pCmdBuf;
    DWORD  CmdSize          =0;
    DWORD  AllocOffset      =0;
  
    zx_dev_t *dev          =(zx_dev_t*)pdev;
    
    counter_t *pcounter    =&(dev->counter);

    DWORD  allocation       =dev->bo->allocation;
    DWORD  CounterSize      =pcounter->CounterSize;
    DWORD  ILACounterMode   =pcounter->mode;
    DWORD* pData;

    Reg_Csp_Misc_Control     reg_Csp_Misc_Control      ={0};

    DWORD  counterval[HWINFO_COUTER_SIZE]={0};  //row counter
    DWORD  counter[HWINFO_COUTER_SIZE]={0};    // diff counter
    unsigned long long      timestamp=0;
    unsigned int            eclk=dev->eclk;
    static DWORD fenceid=0;
 
    map_args_t              arg                  = { 0};
    int i=0;
    zx_perf_status_t status;
    if(eclk==0)
    {
        eclk=300;
    }
     
    CmdSize += 16;
   
    pCmdBuf= zx_context_request_space(dev->renderContext, CmdSize ,ZX_CMD_HINT_3DBLT);

    pcounter->virt_addr=NULL;
    

    zx_bo_wait_idle(dev->bo);

    zx_bo_map(dev->bo, &arg);

    pcounter->virt_addr = arg.virt_addr;
    memset(pcounter->virt_addr,0,4096*sizeof(DWORD));
    zx_bo_unmap(dev->bo);    

    fenceid++;
    AllocOffset=0;

    zx_context_add_allocation(dev->renderContext,allocation,pCmdBuf+1,AllocOffset, 0, 1, 1, MM_DRIVERID_VIDEO_RESERVEDMEM);
       
    pCmdBuf += SEND_VCP_QUERY_E3K(VCP_QUERY_DUMP_BLOCK_ID_VCP_FE, Reg_Fe_Framelvl_Cmp_Memcount_Offset, 
                                      S3VD_FE_CMP_COUNTER_SIZE, (large_inter)(dev->bo->gpu_virt_addr+AllocOffset), pCmdBuf);
    
    pCmdBuf +=SEND_INTERNAL_FENCE(VCP_QUERY_DUMP_BLOCK_ID_VCP_FE,HWM_SYNC_VCP0_FE_SLOT,fenceid,pCmdBuf);

    AllocOffset+=S3VD_FE_CMP_COUNTER_SIZE*sizeof(DWORD);

    zx_context_add_allocation(dev->renderContext,allocation,pCmdBuf+1,AllocOffset, 0, 1, 1, MM_DRIVERID_VIDEO_RESERVEDMEM);

    pCmdBuf += SEND_VCP_QUERY_E3K(VCP_QUERY_DUMP_BLOCK_ID_VCP_BE, Reg_Be_Framelvl_Cmp_Memcount_Offset,
    S3VD_BE_CMP_COUNTER_SIZE, (large_inter)(dev->bo->gpu_virt_addr+AllocOffset), pCmdBuf);
 
    pCmdBuf +=SEND_INTERNAL_FENCE(VCP_QUERY_DUMP_BLOCK_ID_VCP_BE,HWM_SYNC_VCP0_BE_SLOT,fenceid,pCmdBuf);
   
    zx_context_release_space(dev->renderContext, pCmdBuf);

    zx_context_flush(dev->renderContext);

    zx_bo_wait_idle(dev->bo);



    zx_read_reg(fd, MMIO_Csp_Ref_Total_Gpu_Timestamp_Offset, MS_COUNTER_SIZE, &counterval[0]);

    zx_read_reg(fd, MMIO_Ila_Counters_Offset, ILA_COUNTER_SZIE, &counterval[MS_COUNTER_SIZE]);

    read_mxu1_counter(fd, &counterval[MS_COUNTER_SIZE + ILA_COUNTER_SZIE]);

    arg.virt_addr=NULL;
    zx_bo_map(dev->bo, &arg);

    pcounter->virt_addr = arg.virt_addr;

    if(pcounter->virt_addr)
    {
        pData=(DWORD *)pcounter->virt_addr;

        int offset=MS_COUNTER_SIZE + ILA_COUNTER_SZIE + MXU1_COUNTER_SZIE;

        for(int i=0;i<S3VD_COUNTER_SIZE;i++)
        {
            counterval[i+offset]=*(pData+i);
        }

    }

    if(pmeminfo)
    {    
        timestamp=*(unsigned long long*)counterval - *(unsigned long long*)(pcounter->counterval);

#if DEBUGCOUNTER
        printf("timestamp=%llu\n",timestamp);
#endif

        for(int i=MS_COUNTER_SIZE;i<HWINFO_COUTER_SIZE;i++)
        {
          
            counter[i]=(counterval[i]>=pcounter->counterval[i]) ? (counterval[i]-pcounter->counterval[i]) : (0xffffffff-pcounter->counterval[i]+counterval[i]);                 
            
        }


        // counter in memory layout : ms_counter, ila_counter, mxu1_counter, s3vd_counter
        pmeminfo->diu_rd_db=counter[CSP_COUNTER_SIZE]/8*256llu/(timestamp/27);

        pmeminfo->diu_wr_db=0;

      
        pmeminfo->local_rd_db=counter[MS_COUNTER_SIZE + 13]/8*256llu/(timestamp/27) + pmeminfo->diu_rd_db;

        pmeminfo->local_wr_db=counter[MS_COUNTER_SIZE + 14]/8*256llu/(timestamp/27);


        pmeminfo->vcp_rd_db=(counter[CSP_COUNTER_SIZE + MXU1_COUNTER_SZIE]/8*256llu/(timestamp/27))+(counter[CSP_COUNTER_SIZE + MXU1_COUNTER_SZIE + 2]/8*256llu/(timestamp/27)) //vcp 1
                             +counter[MS_COUNTER_SIZE + 9]/8*256llu/(timestamp/27) + counter[MS_COUNTER_SIZE + 21]/8*256llu/(timestamp/27);    // vcp0

        pmeminfo->vcp_wr_db=(counter[CSP_COUNTER_SIZE + MXU1_COUNTER_SZIE + 1]/8*256llu/(timestamp/27))+(counter[CSP_COUNTER_SIZE + MXU1_COUNTER_SZIE + 3]/8*256llu/(timestamp/27)) //vcp 1
                             +counter[MS_COUNTER_SIZE + 10]/8*256llu/(timestamp/27) + counter[MS_COUNTER_SIZE + 24]/8*256llu/(timestamp/27);    // vcp0
        
        pmeminfo->vpp_rd_db= counter[MS_COUNTER_SIZE + 28]/8*256llu/(timestamp/27);
        pmeminfo->vpp_wr_db= counter[MS_COUNTER_SIZE + 29]/8*256llu/(timestamp/27);
        pmeminfo->gfx_rd_db=pmeminfo->local_rd_db - pmeminfo->diu_rd_db - pmeminfo->vcp_rd_db - pmeminfo->vpp_rd_db;

        pmeminfo->gfx_wr_db=pmeminfo->local_wr_db - pmeminfo->diu_wr_db -pmeminfo->vcp_wr_db - pmeminfo->vpp_wr_db;

        pmeminfo->pcie_rd_db= counter[CSP_COUNTER_SIZE + 1 ]/8*256llu/(timestamp/27) + counter[CSP_COUNTER_SIZE + 4]/8*256llu/(timestamp/27);
	
        pmeminfo->pcie_wr_db= counter[CSP_COUNTER_SIZE + 2]/8*256llu/(timestamp/27) + counter[CSP_COUNTER_SIZE + 3]/8*256llu/(timestamp/27);
    
    }

    memset(&status, 0, sizeof(zx_perf_status_t));
    zxGetPerfStatus(fd, &status);

    if(dev->addperfevent && pmeminfo && status.started)
    {

        zx_perf_event_t perf_event;
        zx_perf_event_bandwidth_t *event = (zx_perf_event_bandwidth_t *) &perf_event;
        unsigned long long ts;

        memset(&perf_event, 0, sizeof(zx_perf_event_t));

        event->header.size = sizeof(zx_perf_event_bandwidth_t);
        event->header.type = ZX_PERF_EVENT_BANDWIDTH;
        event->header.pid = zx_context_get_current_pid();
        event->header.tid =  zx_context_get_current_tid();
        ts = zx_context_get_current_system_time();
        event->header.timestamp_low = ts & 0xffffffff;
        event->header.timestamp_high = ts >> 32;
        memcpy(&event->bandwidth_data, pmeminfo, sizeof(zx_meminfo));

        zxSendPerfEvent(fd, &perf_event);
    }

    for(int i=0;i<HWINFO_COUTER_SIZE;i++)
    {     
        pcounter->counterval[i]=counterval[i];
    }
    pcounter->gputimestamp=timestamp;   

    zx_bo_unmap(dev->bo);




    

    return 0;

}



 void gfx_deinit_hwinfo(int fd, void *pdev)
 {
    zx_dev_t *dev          =(zx_dev_t*)pdev;

    zx_bo_unreference(dev->bo);
    zx_destroy_context(dev->renderContext);
    zxDestroyDevice(fd,dev->zxDevice);
 }

