/*
 * Copyright (c) 2018 Shanghai Zhaoxin Semiconductor Co., Ltd.
 * All Rights Reserved.
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Shanghai Zhaoxin Semiconductor Co., Ltd.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Shanghai Zhaoxin Semiconductor Co., Ltd.
 *
 * The copyright of the source code is protected by the copyright laws of the People's
 * Republic of China and the related laws promulgated by the People's Republic of China
 * and the international covenant(s) ratified by the People's Republic of China.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#define stat_t struct stat
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/time.h>
#include <stdarg.h>
#include <assert.h>
#include <libdrm/drm.h>
#include <xf86drm.h>

#include "zx_keinterface.h"
#include "zx_ioctl.h"

#ifdef ZX_VMI_MODE

#ifdef ioctl
#undef ioctl
#endif

#define ioctl zx_ioctl
extern int zx_ioctl(int fd, unsigned long request, ...);

#endif


#ifndef MAP_FAILED
#define MAP_FAILED ((void *)-1)
#endif

#define ZX_DEV_UID         0
#define ZX_DEV_GID         0

#define ZX_DEV_DIRMODE             777
//(S_IXOTH | S_IXUSR | S_IXGRP | S_ISUID | S_ISGID | S_IWOTH | S_IROTH | S_IWGRP | S_IRGRP | S_IWUSR | S_IRUSR)
#define ZX_DEV_MODE         (S_IWOTH | S_IROTH | S_IWGRP | S_IRGRP | S_IWUSR | S_IRUSR)


struct drm_version_zx{
    int major;
    int minor;
    int patchlevel;
    size_t name_len;
    char* name;
    size_t date_len;
    char* date;
    size_t desc_len;
    char* desc;
};

int zxCreateContext(int fd, zx_create_context_t *create)
{
    if(ioctl(fd, ZX_IOCTL_CREATE_CONTEXT, create))
    {
        return -errno;
    }

    return 0;
}

int zxDestroyContext(int fd, zx_destroy_context_t *destroy)
{
    if(ioctl(fd, ZX_IOCTL_DESTROY_CONTEXT, destroy))
    {
        return -errno;
    }

    return 0;
}

int zxRender(int fd, zx_render_t *render)
{
    if(ioctl(fd, ZX_IOCTL_RENDER, render))
    {
        return -errno;
    }

    return 0;
}

int zxCreateDevice(int fd, zx_create_device_t* create_device)
{   
    if(ioctl(fd, ZX_IOCTL_CREATE_DEVICE, create_device))
    {
        return -errno;
    }

    return 0;
}

int zxDestroyDevice(int fd, unsigned int device)
{
    if(ioctl(fd, ZX_IOCTL_DESTROY_DEVICE, &device))
    {
        return -errno;
    }

    return 0;
}

int zxWaitChipIdle(int fd)
{
    if(ioctl(fd, ZX_IOCTL_WAIT_CHIP_IDLE))
    {
        return -errno;
    }

    return 0;
}

int zxWaitAllocationIdle(int fd, zx_wait_allocation_idle_t *wait_allocation)
{
    if(ioctl(fd, ZX_IOCTL_WAIT_ALLOCATION_IDLE, wait_allocation))
    {
        return -errno;
    }
   
    return 0;
}

int zxQueryInfo(int fd, zx_query_info_t *info)
{
    if(ioctl(fd, ZX_IOCTL_QUERY_INFO, info))
    {
        return -errno;
    }

    return 0;
}

int zxBeginPerfEvent(int fd, zx_begin_perf_event_t *begin_perf_event)
{
    if(ioctl(fd, ZX_IOCTL_BEGIN_PERF_EVENT, begin_perf_event))
    {
        return -errno;
    }

    return 0;
}

int zxEndPerfEvent(int fd, zx_end_perf_event_t *end_perf_event)
{
    if(ioctl(fd, ZX_IOCTL_END_PERF_EVENT, end_perf_event))
    {
        return -errno;
    }

    return 0;
}

int zxGetPerfEvent(int fd, zx_get_perf_event_t *get_perf_event)
{
    if(ioctl(fd, ZX_IOCTL_GET_PERF_EVENT, get_perf_event))
    {
        return -errno;
    }

    return 0;
}

int zxSendPerfEvent(int fd, zx_perf_event_t *perf_event)
{
    if(ioctl(fd, ZX_IOCTL_SEND_PERF_EVENT, perf_event))
    {
        return -errno;
    }

    return 0;
}

int zxGetPerfStatus(int fd, zx_perf_status_t *perf_status)
{
    if(ioctl(fd, ZX_IOCTL_GET_PERF_STATUS, perf_status))
    {
        return -errno;
    }

    return 0;
}

int zxBeginMiuDumpPerfEvent(int fd, zx_begin_miu_dump_perf_event_t *begin_miu_perf_event)
{
    if(ioctl(fd, ZX_IOCTL_BEGIN_MIU_DUMP_PERF_EVENT, begin_miu_perf_event))
    {
        return -errno;
    }

    return 0;
}

int zxEndMiuDumpPerfEvent(int fd, zx_end_miu_dump_perf_event_t *end_miu_perf_event)
{
    if(ioctl(fd, ZX_IOCTL_END_MIU_DUMP_PERF_EVENT, end_miu_perf_event))
    {
        return -errno;
    }

    return 0;
}

int zxSetMiuRegListPerfEvent(int fd, zx_miu_reg_list_perf_event_t *miu_reg_list)
{
    if(ioctl(fd, ZX_IOCTL_SET_MIU_REG_LIST_PERF_EVENT, miu_reg_list))
    {
        return -errno;
    }

    return 0;
}

int zxGetMiuDumpPerfEvent(int fd, zx_get_miu_dump_perf_event_t *get_miu_dump)
{
    if(ioctl(fd, ZX_IOCTL_GET_MIU_DUMP_PERF_EVENT, get_miu_dump))
    {
        return -errno;
    }

    return 0;
}

int zxDirectGetMiuDumpPerfEvent(int fd, zx_direct_get_miu_dump_perf_event_t *direct_get_dump)
{
    if(ioctl(fd, ZX_IOCTL_DIRECT_GET_MIU_DUMP_PERF_EVENT, direct_get_dump))
    {
        return -errno;
    }

    return 0;
}


int zxDvfsSet(int fd, zx_dvfs_set_t * set)
{
    if (ioctl(fd, ZX_IOCTL_DVFS_SET, set))
    {
        return -errno;
    }

    return 0;
}

int zxQueryDvfsClamp(int fd, zx_dvfs_clamp_status_t * dvfs_clamp)
{
    if (ioctl(fd, ZX_IOCTL_QUERY_DVFS_CLAMP, dvfs_clamp))
    {
        return -errno;
    }

    return 0;
}

int zxCreateFenceSyncObject(int fd, zx_create_fence_sync_object_t *create)
{
    if(ioctl(fd, ZX_IOCTL_CREATE_FENCE_SYNC_OBJECT, create))
    {
        return -errno;
    }

    return 0;
}

int zxDestroyFenceSyncObject(int fd, zx_destroy_fence_sync_object_t *destroy)
{
    if(ioctl(fd, ZX_IOCTL_DESTROY_FENCE_SYNC_OBJECT, destroy))
    {
        return -errno;
    }

    return 0;
}

int zxWaitFenceSyncObject(int fd, zx_wait_fence_sync_object_t *wait)
{
    if(ioctl(fd, ZX_IOCTL_WAIT_FENCE_SYNC_OBJECT, wait))
    {
        return -errno;
    }

    return 0;
}

int zxFenceValue(int fd, zx_fence_value_t *fence)
{
    if(ioctl(fd, ZX_IOCTL_FENCE_VALUE, fence))
    {
        return -errno;
    }

    return 0;
}

int zxWaitContextIdle(int fd, unsigned int context)
{
    if (ioctl(fd, ZX_IOCTL_WAIT_CONTEXT_IDLE, &context))
    {
        return -errno;
    }
    return 0;
}

int zxGetAllocationState(int fd, zx_get_allocation_state_t *state)
{
    if (ioctl(fd, ZX_IOCTL_GET_ALLOCATION_STATE, state))
    {
        return -errno;
    }

    return 0;
}


int zxGemCreateAllocation(int fd, zx_create_allocation_t *create)
{
    if(ioctl(fd, ZX_IOCTL_DRM_GEM_CREATE_ALLOCATION, create))
    {
        return -errno;
    }

    return 0;
}

int zxGemDestroyObject(int fd, unsigned int handle)
{
    struct drm_gem_close close={0};

    close.handle = handle;

#ifdef ZX_VMI_MODE
    ioctl(fd, DRM_IOCTL_GEM_CLOSE, &close);
#else
    drmIoctl(fd, DRM_IOCTL_GEM_CLOSE, &close);
#endif

    return 0;
}

int zxGemCreateResource(int fd, zx_create_resource_t *create)
{
    if(ioctl(fd, ZX_IOCTL_DRM_GEM_CREATE_RESOURCE, create))
    {
        return -errno;
    }

    return 0;
}

int zxGemBeginCpuAccess(int fd, zx_drm_gem_begin_cpu_access_t *begin)
{
    if (ioctl(fd, ZX_IOCTL_DRM_BEGIN_CPU_ACCESS, begin))
    {
        return -errno;
    }

    return 0;
}

int zxGemEndCpuAccess(int fd, zx_drm_gem_end_cpu_access_t *end)
{
    if (ioctl(fd, ZX_IOCTL_DRM_END_CPU_ACCESS, end))
    {
        return -errno;
    }

    return 0;
}

int zxGemMapGtt(int fd, zx_drm_gem_map_t *map)
{
    if(ioctl(fd, ZX_IOCTL_DRM_GEM_MAP_GTT, map))
    {
        return -errno;
    }

    return 0;
}

int zxGemUnmap(int fd)
{
    assert(0);

    return 0;
}



int zxAddHwCtxBuf(int fd, zx_add_hw_ctx_buf_t *add)
{
    if(ioctl(fd, ZX_IOCTL_ADD_HW_CTX_BUF, add))
    {
        return -errno;
    }

    return 0;
}

int zxRmHwCtxBuf(int fd, zx_rm_hw_ctx_buf_t *rm)
{
    if(ioctl(fd, ZX_IOCTL_RM_HW_CTX_BUF, rm))
    {
        return -errno;
    }

    return 0;
}

int zxCIL2Misc(int fd,zx_cil2_misc_t * misc)
{
    if( ioctl(fd, ZX_IOCTL_CIL2_MISC, misc) )
    {
        return -errno;
    }

    return 0;
}



int zxGetPipeFromCRTC(int fd, zx_kms_get_pipe_from_crtc_t *get_pipe)
{
    if (ioctl(fd, ZX_IOCTL_KMS_GET_PIPE_FROM_CRTC, get_pipe))
    {
        return -errno;
    }

    return 0;
}

int zxDelayInitCard(int fd, zx_delay_init_card_t *delay_init_card)
{
    if (ioctl(fd, ZX_IOCTL_DELAY_INIT_CARD, delay_init_card))
    {
        return -errno;
    }

    return 0;
}

static struct kernel_interface kinterface_struct_v2 = {
    .WaitChipIdle = zxWaitChipIdle,
    .WaitAllocationIdle = zxWaitAllocationIdle,
    .QueryInfo = zxQueryInfo,
    .CreateDevice = zxCreateDevice,
    .DestroyDevice = zxDestroyDevice,
    .BeginPerfEvent = zxBeginPerfEvent,
    .EndPerfEvent = zxEndPerfEvent,
    .GetPerfEvent = zxGetPerfEvent,
    .SendPerfEvent = zxSendPerfEvent,
    .GetPerfStatus = zxGetPerfStatus,
    .BeginMiuDumpPerfEvent = zxBeginMiuDumpPerfEvent,
    .EndMiuDumpPerfEvent = zxEndMiuDumpPerfEvent,
    .SetMiuRegListPerfEvent = zxSetMiuRegListPerfEvent,
    .GetMiuDumpPerfEvent = zxGetMiuDumpPerfEvent,
    .DirectGetMiuDumpPerfEvent = zxDirectGetMiuDumpPerfEvent,
    .DvfsSet = zxDvfsSet,
    .QueryDvfsClamp = zxQueryDvfsClamp,    
    .CreateContext = zxCreateContext,
    .DestroyContext = zxDestroyContext,
    .Render = zxRender,
    .CreateFenceSyncObject = zxCreateFenceSyncObject,
    .DestroyFenceSyncObject = zxDestroyFenceSyncObject,
    .WaitFenceSyncObject = zxWaitFenceSyncObject,
    .FenceValue = zxFenceValue,
    .WaitContextIdle = zxWaitContextIdle,
    .AddHwCtxBuf = zxAddHwCtxBuf,
    .RmHwCtxBuf = zxRmHwCtxBuf,
    .CIL2Misc = zxCIL2Misc,
    .GemDestroyObject = zxGemDestroyObject,
    .GemCreateAllocation = zxGemCreateAllocation,
    .GemCreateResource = zxGemCreateResource,
    .GemMapGtt = zxGemMapGtt,
    .GemUnmap = zxGemUnmap,
    .GetAllocationState = zxGetAllocationState,
    .GetPipeFromCRTC = zxGetPipeFromCRTC,
    .GemBeginCpuAccess = zxGemBeginCpuAccess,
    .GemEndCpuAccess = zxGemEndCpuAccess,
    .DelayInitCard = zxDelayInitCard,
};

__attribute__((visibility("default")))
struct kernel_interface* CONCAT(kinterface_v2,DRIVER_NAME) = &kinterface_struct_v2;
