/*
 * Copyright (c) 2018 Shanghai Zhaoxin Semiconductor Co., Ltd.
 * All Rights Reserved.
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Shanghai Zhaoxin Semiconductor Co., Ltd.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Shanghai Zhaoxin Semiconductor Co., Ltd.
 *
 * The copyright of the source code is protected by the copyright laws of the People's
 * Republic of China and the related laws promulgated by the People's Republic of China
 * and the international covenant(s) ratified by the People's Republic of China.
 *
 */
#include "zx.h"
#include "zx_driver.h"
#include "os_interface.h"
#include <linux/of.h>
#include <linux/of_device.h>

static int zx_ahb_probe(struct platform_device *pdev)
{
    zx_card_t *zx = NULL;
    int         result;

    zx_info("zx_ahb_probe\n");

    zx_memset(zx, 0, sizeof(zx_card_t));

    if(zx_fb_mode == NULL)
    {
        zx_fb_mode = "1920x1080-32@60";//1080p by default
    }

    result = zx_card_init(zx, pdev);

    if(result == 0)
    {
        platform_set_drvdata(pdev, zx);
    }
   return result;
}

static int zx_ahb_remove(struct platform_device *pdev)
{
    zx_card_t *zx = platform_get_drvdata(pdev);

    if(zx == NULL) return -1;

    zx_card_deinit(zx);
    zx_free(zx);
    platform_set_drvdata(pdev, NULL);

    return 0;
}

static int zx_ahb_suspend(struct platform_device *pdev, pm_message_t state)
{
    zx_card_t* zx = platform_get_drvdata(pdev);

//    zx_info("zx driver suspending, pm event = %d.\n", state.event);
#ifdef CONFIG_FB
    zx_fb_set_pm(zx, 1);
#endif

    krnl_save_state(zx->adapter, 1);

    /* disable IRQ */
    zx_interrupt_deinit(zx);

    //platform_driver_to_do:

    return 0;
}

static int zx_ahb_resume(struct platform_device *pdev)
{
    zx_card_t* zx = platform_get_drvdata(pdev);
    int         ret = 0;
    long        time_sec = 0, time_usec = 0;

    //temp use only, will remove after resume stable.
    zx_getsecs(&time_sec, &time_usec);
    zx_info("zx driver resume back, cur time %ld(ms)\n", (time_sec * 1000 + zx_do_div(time_usec, 1000)));

    zx_interrupt_reinit(zx); 

    ret = krnl_restore_state(zx->adapter);

    if(ret != 0)
    {
        return -1;
    }

#ifdef CONFIG_FB
    zx_fb_set_pm(zx, 0);
#endif

    //temp use only, will remove after resume stable.
    zx_getsecs(&time_sec, &time_usec);
    zx_info("zx driver resume done, cur time %ld(ms)\n", (time_sec * 1000 + zx_do_div(time_usec, 1000)));

    return 0;
}

#ifdef CONFIG_OF
#if defined(ELT1K) || defined(ELT2K)
static struct platform_device_id general_gfx_driver_ids[] = {
    {
        .name = "elite-gfx",
        .driver_data = (kernel_ulong_t)NULL,
    },
    {
        .name = "elite-gfx.0",
        .driver_data = (kernel_ulong_t)NULL,
    },
};

static const struct of_device_id general_gfx_match[] = {
    {
#ifdef ELT2K
        .compatible = "zxraphics,elite2000-gfx"
#else
        .compatible = "zxraphics,elite1000-gfx"
#endif
    },
};
#endif

#ifdef ZX2K
static struct platform_device_id general_gfx_driver_ids[] = {
    {
        .name = "zx-gfx",
        .driver_data = (kernel_ulong_t)NULL,
    },
    {
        .name = "zx-gfx.0",
        .driver_data = (kernel_ulong_t)NULL,
    },
};

static const struct of_device_id general_gfx_match[] = {
    {
        .compatible = "zhaoxin,zx2000-gfx"
    },
};
#endif

#endif

struct platform_driver zx_driver = {
    .probe    = zx_ahb_probe,
    .remove   = zx_ahb_remove,
    .suspend  = zx_ahb_suspend,
    .resume   = zx_ahb_resume,
#ifdef CONFIG_OF
    .id_table = general_gfx_driver_ids,
#endif
    .driver   = {
#ifdef ZX2K
        .name = "zx-gfx",
#else
        .name = "elite-gfx",
#endif
        .owner = THIS_MODULE,
#ifdef CONFIG_OF
    .of_match_table = of_match_ptr(general_gfx_match),
#endif
    },
};

/* for elite in ahb bus, only use it to check if current card is primary(0x01) */
int zx_get_command_status16(void *dev, unsigned short *command)
{
    *command = 0x01;
    return 0;
}

int zx_get_command_status32(void *dev, unsigned int *command)
{
    *command = 0x01;
    return 0;
}

/* useless for elite in ahb bus */
int zx_write_command_status16(void *dev, unsigned short command)
{
    return 0;
}

/* useless for elite in ahb bus */
int zx_write_command_status32(void *dev, unsigned int command)
{
    return 0;
}

/* useless for elite in ahb bus */
int zx_get_bar1(void *dev, unsigned int *bar1)
{
    *bar1 = 0x0;
    return 0;
}

/* useless for elite in ahb bus */
int zx_get_rom_save_addr(void *dev, unsigned int *romsave)
{
    *romsave = 0x0;
    return 0;
}

/* useless for elite in ahb bus */
int zx_write_rom_save_addr(void *dev, unsigned int romsave)
{
    return 0;
}


int zx_get_platform_config(void *dev, const char* config_name, int *buffer, int length)
{
    struct platform_device *pdev = (struct platform_device *)dev;
    struct device_node *np = pdev->dev.of_node;

    return of_property_read_u32_array(np, config_name, buffer, length);
}

void zx_get_bus_config(void *dev, bus_config_t *bus)
{
    struct platform_device *pdev = (struct platform_device *)dev;
    struct device_node *np = pdev->dev.of_node;
    int value[2] = {0};

//    bus->device_id         = 0x320f;
//    bus->vendor_id         = 0x5333;
    bus->command           = 0x01;
    bus->status            = 0;
    bus->revision_id       = 0;
    bus->prog_if           = 0;
    bus->sub_class         = 0;
    bus->base_class        = 0;
    bus->cache_line_size   = 0;
    bus->latency_timer     = 0;
    bus->header_type       = 0;
    bus->bist              = 0;
    bus->sub_sys_vendor_id = 0;
    bus->sub_sys_id        = 0;
    bus->link_status       = 0;

    of_property_read_u32_array(np, "reg,mmio", value, 2);

    bus->reg_start_addr[0] = value[0];
    bus->reg_end_addr[0] = value[0] + value[1];
    value[0] = 0;
    value[1] = 0;

    of_property_read_u32_array(np, "reg,pmc", value, 2);

    bus->reg_start_addr[1] = value[0];
    bus->reg_end_addr[1] = value[0] + value[1];
    value[0] = 0;
    value[1] = 0;

    of_property_read_u32_array(np, "reg,gpio", value, 2);

    bus->reg_start_addr[2] = value[0];
    bus->reg_end_addr[2] = value[0] + value[1];
    value[0] = 0;
    value[1] = 0;

    of_property_read_u32_array(np, "reg,pmu", value, 2);

    bus->reg_start_addr[3] = value[0];
    bus->reg_end_addr[3] = value[0] + value[1];
    value[0] = 0;
    value[1] = 0;

#ifdef ELT2K

    of_property_read_u32_array(np, "mem,lcs", value, 2);

    bus->mem_start_addr[1] = value[0];
    bus->mem_end_addr[1] = value[0] + value[1];
    value[0] = 0;
    value[1] = 0;

    of_property_read_u32_array(np, "mem,local", value, 2);

    bus->mem_start_addr[2] = value[0];
    bus->mem_end_addr[2] = value[0] + value[1];
    value[0] = 0;
    value[1] = 0;

    of_property_read_u32_array(np, "mem,secure", value, 2);

    bus->secure_start_addr[0] = value[0];
    bus->secure_end_addr[0] = value[0] + value[1];

#else
#ifdef ELT1K
    of_property_read_u32_array(np, "mem,secure2", value, 2);

    bus->secure_start_addr[0] = value[0];
    bus->secure_end_addr[0] = value[0] + value[1];
    value[0] = 0;
    value[1] = 0;

    of_property_read_u32_array(np, "mem,secure3", value, 2);

    bus->secure_start_addr[1] = value[0];
    bus->secure_end_addr[1] = value[0] + value[1];
#endif

#ifdef ZX2K
    of_property_read_u32_array(np, "mem,secure", value, 2);

    bus->secure_start_addr[0] = value[0];
    bus->secure_end_addr[0] = value[0] + value[1];
    value[0] = 0;
    value[1] = 0;

    of_property_read_u32_array(np, "mem,local", value, 2);

    bus->mem_start_addr[1] = value[0];
    bus->mem_end_addr[1] = value[0] + value[1];
    value[0] = 0;
    value[1] = 0;

#endif
#endif

    bus->secure_on = svc_system_is_secureboot();
  
    {
    unsigned char *mmio = zx_ioremap(bus->reg_start_addr[0], bus->reg_end_addr[0] - bus->reg_start_addr[0]);

    if(mmio != NULL)
    {
        bus->device_id   = (zx_read32(mmio + 0x8510) & 0xffff0000) >> 16;
        bus->vendor_id   = zx_read32(mmio + 0x8510) & 0xffff;
        bus->revision_id = zx_read32(mmio + 0x8500) & 0x0f;

#ifndef ZX2K
        if(bus->revision_id < 2)
        {
            bus->device_id = 0x3200;
            bus->vendor_id = 0x1166;
        }
#endif
#ifdef ELT2K
        bus->device_id = 0x330F;//elite2000
        bus->vendor_id = 0x5333;
#endif
        zx_iounmap(mmio);
    }
    else
    {
        zx_error("map io address failed: phys start: %x, phys end: %x.\n", bus->reg_start_addr[0], bus->reg_end_addr[0]);
    }

    }

}
/* useless for elite in ahb bus */
unsigned long zx_get_rom_start_addr(void *dev)
{
    return 0;
}

/* useless for elite in ahb bus */
void zx_init_bus_id(zx_card_t *zx)
{
    zx->len = snprintf(zx->busId, 40, "ahb:%04x:%02x:%02x.%d", 0, 0, 0, 0);
}

int zx_register_driver(void)
{
    return platform_driver_register(&zx_driver);
}

void zx_unregister_driver(void)
{
    platform_driver_unregister(&zx_driver);
}

int zx_register_interrupt(zx_card_t *zx, void *isr)
{
    struct resource *interrupt = platform_get_resource(zx->pdev, IORESOURCE_IRQ, 0);

    int ret = 0;

    zx_info("zx interrupt irq %d\n", interrupt->start);
    ret = request_irq(interrupt->start, (irqreturn_t (*)(int, void*))isr, IRQF_SHARED, "zx", zx);
#if !defined(ELT2K) && !defined(ZX2K)
    if (!ret)
    {
        interrupt = platform_get_resource(zx->pdev, IORESOURCE_IRQ, 2);

        ret = request_irq(interrupt->start, (irqreturn_t (*)(int, void*))isr, IRQF_SHARED, "zx", zx);
    }
#endif
    return ret;

}

void zx_unregister_interrupt(zx_card_t *zx)
{
    struct resource *interrupt = platform_get_resource(zx->pdev, IORESOURCE_IRQ, 0);

    free_irq(interrupt->start, zx);

#ifndef ELT2K
    interrupt = platform_get_resource(zx->pdev, IORESOURCE_IRQ, 2);

    free_irq(interrupt->start, zx);
#endif
}
