/*
 * Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
 * Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
 *
 * This file is part of zx.ko
 *
 * zx.ko is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * zx.ko is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "zx_disp.h"
#include "zx_vip.h"
#include "zx_wb.h"
#include "zx_capture_drv.h"

static disp_info_t *g_disp_info = NULL;

void disp_register_capture(disp_info_t *disp_info, zx_capture_t *capture)
{
    capture->cf_next = disp_info->captures;
    disp_info->captures  = capture;
}

void disp_unregister_capture(disp_info_t *disp_info, zx_capture_t *capture)
{
    zx_capture_t **actcf;

    for (actcf = &disp_info->captures; *actcf && *actcf != capture; actcf = &((*actcf)->cf_next))
        ;

    if (*actcf)
        *actcf = (*actcf)->cf_next;
}


zx_capture_t *disp_find_capture(disp_info_t *disp_info, int id)
{

    zx_capture_t *actcf;

    for (actcf = disp_info->captures; actcf && actcf->cf_fmt_id != id; actcf = actcf->cf_next)
        ;

    return actcf;
}


int disp_capture_ctl(void *pdata, zx_capture_id_t id, void *params)
{
    disp_info_t *disp_info = (disp_info_t *)pdata;

    zx_capture_t *capture = disp_find_capture(disp_info, id);


    if (!capture)
    {
        zx_assert(0, "wrong capture fmt id !!!\n");
    }

    return capture->cf_ctl(disp_info, capture, params);
}


void disp_capture_init(disp_info_t *disp_info)
{
    zx_info("disp cature init enter \n");
    disp_register_vip_capture(disp_info);
    disp_register_wb_capture(disp_info);

    //TODO: refine the card index function
    g_disp_info = disp_info;

    zx_info("disp cature init out \n");
}

void disp_capture_deinit(disp_info_t *disp_info)
{
    zx_info("disp cature deinit enter \n");
    disp_unregister_vip_capture(disp_info);
    disp_unregister_wb_capture(disp_info);

    //TODO: refine the card index function
    g_disp_info = NULL;

    zx_info("disp cature deinit out \n");
}


#define ZX_CF_INT_TBL_ENTRY(_cf_id, _irq_mask)       \
    {                                                \
        .cf_id    = _cf_id,                          \
        .irq_mask = _irq_mask,                       \
    }

static zx_cf_irq_tbl_t cf_irq_tbl[] =
{
    ZX_CF_INT_TBL_ENTRY(ZX_CAPTURE_VIP1, INT_VIP1),
    ZX_CF_INT_TBL_ENTRY(ZX_CAPTURE_VIP2, INT_VIP2),
    ZX_CF_INT_TBL_ENTRY(ZX_CAPTURE_VIP3, INT_VIP3),
    ZX_CF_INT_TBL_ENTRY(ZX_CAPTURE_VIP4, INT_VIP4),
    ZX_CF_INT_TBL_ENTRY(ZX_CAPTURE_WB1,  INT_VSYNC1),
    ZX_CF_INT_TBL_ENTRY(ZX_CAPTURE_WB2,  INT_VSYNC2),
    ZX_CF_INT_TBL_ENTRY(ZX_CAPTURE_WB3,  INT_VSYNC3),
    ZX_CF_INT_TBL_ENTRY(ZX_CAPTURE_WB4,  INT_VSYNC4),
};

void zx_capture_handle(disp_info_t *disp_info, unsigned int itrr)
{
    zx_capture_t *capture = NULL;
    int i = 0;

    for (; i < sizeof(cf_irq_tbl) / sizeof(zx_cf_irq_tbl_t); i++)
    {
        if (itrr & cf_irq_tbl[i].irq_mask)
        {
            capture = disp_find_capture(disp_info, cf_irq_tbl[i].cf_id);
            if (capture != NULL && (capture->flags & CAPTURE_ENABLE_B))
            {
                capture->notify_interrupt(disp_info, capture);
            }
        }

    }

}


void zx_krnl_get_capture_interface(unsigned int id, void **pdata, PFN_ZX_CAPTURE_OPS_T *interface)
{
    *pdata = (void *)(g_disp_info);
    *interface = disp_capture_ctl;
}

EXPORT_SYMBOL(zx_krnl_get_capture_interface);
