/*
 * Copyright (c) 2018 Shanghai Zhaoxin Semiconductor Co., Ltd.
 * All Rights Reserved.
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Shanghai Zhaoxin Semiconductor Co., Ltd.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Shanghai Zhaoxin Semiconductor Co., Ltd.
 *
 * The copyright of the source code is protected by the copyright laws of the People's
 * Republic of China and the related laws promulgated by the People's Republic of China
 * and the international covenant(s) ratified by the People's Republic of China.
 *
 */

#ifndef __ZX_DRIVER_H
#define __ZX_DRIVER_H

#include "zx.h"
#include "zx_def.h"
#include "zx_types.h"
#include "os_interface.h"
#include "kernel_interface.h"
#include "zx_device_debug.h"

typedef struct zx_file zx_file_t;

typedef int (*zx_ioctl_t)(zx_file_t* priv, unsigned int cmd, unsigned long arg);

typedef int (*irq_func_t)(void*);

typedef struct  zx_dvfs_info
{
    unsigned int dvfs_3d_checksize;
    unsigned int dvfs_3d_change_num;
    unsigned int dvfs_vcp0_checksize;
    unsigned int dvfs_vcp0_change_num;
    unsigned int dvfs_vcp1_checksize;
    unsigned int dvfs_vcp1_change_num;
    unsigned int dvfs_vpp_checksize;
    unsigned int dvfs_vpp_change_num;
}zx_dvfs_info_t;

typedef struct
{
    void              *adapter;
    struct pci_dev    *pdev;

    struct drm_device *drm_dev;
    adapter_info_t    adapter_info;
    void*             disp_info;
    char              busId[64];
    int               len;
    int               index;

    struct os_mutex   *lock;

    int               support_msi;

    struct tasklet_struct fence_notify;

    int               zxfb_enable;
    int               reserved_vmem;

    void                *debugfs_dev;
    struct device       *pci_device;
    struct zx_dma_fence_driver *fence_drv;
    void                *fbdev;
    struct backlight_device *backlight_dev;
    bool have_disp_power_ref;
    struct krnl_adapter_init_info_s a_info;
    struct proc_dir_entry *procfs_entry;
    unsigned long       umd_trace_tags; // usermode trace tags, 0 to disable.
    zx_dvfs_info_t      *dvfs_info;
    unsigned long long primary_addr[MAX_CORE_CRTCS];

#define ZX_S4_RESUME 0x01
    unsigned int flags;
}zx_card_t;

static inline struct drm_device *_to_drm_device(void *pdev)
{
    struct drm_device *drm = NULL;

    drm = pci_get_drvdata(pdev);

    return drm;
}

struct zx_file
{
    struct drm_file   *parent_file;
    zx_card_t        *card;

    void              *map;
    unsigned int      gpu_device;

    zx_device_debug_info_t     *debug;

    unsigned int      server_index;

    int               hold_lock;
    int               freezable;
    struct os_mutex   *lock;
};

extern char *zx_fb_mode;
extern int   zx_fb;

extern struct class *zx_class;
extern zx_ioctl_t   zx_ioctls[];
extern zx_ioctl_t   zx_ioctls_compat[];

extern int  zx_card_init(zx_card_t *zx, void *pdev);
extern int  zx_card_deinit(zx_card_t *zx);
extern void zx_card_pre_init(zx_card_t *zx, void *pdev);
extern int  zx_init_modeset(struct drm_device *dev);
extern int  zx_init_psr(struct drm_device *dev);
extern void  zx_deinit_modeset(struct drm_device *dev);
extern void  zx_deinit_psr(struct drm_device *dev);
extern int zx_debugfs_crtc_dump(struct seq_file* file, struct drm_device* dev, int index);
extern int zx_debugfs_clock_dump(struct seq_file* file, struct drm_device* dev);
extern int zx_debugfs_displayinfo_dump(struct seq_file* file, struct drm_device* dev);


extern void zx_interrupt_init(zx_card_t *zx);
extern void zx_interrupt_reinit(zx_card_t *zx);
extern void zx_interrupt_deinit(zx_card_t *zx);

extern int  zx_map_system_io(struct vm_area_struct *vma, zx_map_argu_t *map);
extern int  zx_map_system_ram(struct vm_area_struct *vma, zx_map_argu_t *map);

extern void zx_enable_interrupt(void *pdev);
extern void zx_disable_interrupt(void *pdev);

extern void zx_init_bus_id(zx_card_t *zx);
extern int  zx_register_driver(void);
extern void zx_unregister_driver(void);
extern int  zx_register_interrupt(zx_card_t *zx, void *isr);
extern void zx_unregister_interrupt(zx_card_t *zx);

extern int  zx_create_device(int card, unsigned int *device);
extern void zx_destroy_device(int card, unsigned int device);

extern int zx_ioctl_wait_chip_idle(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_acquire_aperture(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_release_aperture(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_flush_allocation(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_set_interactive(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_wait_allocation_idle(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_video_get_hw_caps(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_query_info(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_create_device(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_destroy_device(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_begin_perf_event(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_begin_miu_dump_perf_event(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_end_perf_event(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_end_miu_dump_perf_event(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_get_perf_event(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_send_perf_event(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_get_perf_status(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_destroy_context(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_wait_context_idle(zx_file_t *priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_query_chip_id(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_hdcp_op(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_create_fence_sync_object(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_destroy_fence_sync_object(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_wait_fence_sync_object(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_fence_value(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_create_fence_fd(zx_file_t* priv, unsigned int cmd, unsigned long arg);

extern int zx_ioctl_wait_context_idle(zx_file_t* priv, unsigned int cmd, unsigned long arg);

extern int zx_ioctl_set_power_perf_mode(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_gfx_stm(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_get_allocation_state(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_mmap(struct file *filp, struct vm_area_struct *vma);
extern int zx_ioctl_cil2_misc(zx_file_t *priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_gem_create_allocation(zx_file_t *priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_gem_create_context(zx_file_t *priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_gem_map_gtt(zx_file_t *priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_gem_begin_cpu_access(zx_file_t *priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_gem_end_cpu_access(zx_file_t *priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_render(zx_file_t *priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_gem_create_resource(zx_file_t *priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_add_hw_ctx_buf(zx_file_t *priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_rm_hw_ctx_buf(zx_file_t *priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_set_miu_reg_list_perf_event(zx_file_t *priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_get_miu_dump_perf_event(zx_file_t *priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_direct_get_miu_dump_perf_event(zx_file_t *priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_dvfs_set(zx_file_t *priv, unsigned cmd, unsigned long arg);
extern int zx_ioctl_query_dvfs_clamp(zx_file_t *priv, unsigned cmd, unsigned long arg);
#endif

